# ==============================================================================
#                base classes used in uiScriptPy module
#  ----------------------------------------------------------------------------
#  Establishes the uiScriptGadget and uiScript container classes
#       base classes for all gadgets including  containers windows etc...
# ==============================================================================

#  LAYOUT ATRIB MUST remain, in abscence access methods
#     Cell atrribs can be on ANY gadgets in a cell layouts container.



import hou

from enums import *
from utils import *

class UiScriptGadget():

	_name        = None # print methods for this ??
	_type        = None
	_value       = None # print methods # different value typees, sing,tuple, vectlist

	_keyvalue    = None
	#_layout      = None # does this belong o nthe container instead
	_hstretch    = 1
	_vstretch    = None
	_width       = None
	_height      = None
	_min_width   = None
	_min_height  = None
	_max_width   = None
	_max_height  = None
	_pref_width  = None
	_pref_height = None
	_hmargin     = None
	_vmargin     = None
	#_spacing     = None
	_hjustify    = None
	_vjustify    = None
	#_cell        = None
	_cell_x      = None
	_cell_y      = None
	#_cellsize    = None
	_cell_width  = None
	_cell_height = None
	_look        = None
	# rmbmenu

	# cmds
	_disable     = False
	_export      = False 
	_open        = False # this is not actually implemented anywhere-- needs a window ref

	# utility (not for attribs)
	_naming_scheme    = uiScriptNamingScheme.Base1
	_num_components   = 1
	_script_callbacks = ()
	

	def name(self):
		return self._name
		
	def type(self):
		return self._type

	def value(self):
		return self._value

		
	def keyvalue(self):
		
		'''Returns keyvalue. value is changed whenever a key is pressed and contains the key value.'''
		
		return self._keyvalue
		

	def stretch(self):
		
		'''Returns the gadget stretchiness values in both directions.
		   0 being none and 1 being normal.'''
		   
		return tuple([self._hstretch, self._vstretch])

	def hStretch(self):
		
		'''Returns the gadget stretchiness value horizontally.
		   0 being none and 1 being normal.'''
		
		return self._hstretch

	def vStretch(self):
		
		'''Returns the gadget stretchiness value vertically.
		   0 being none and 1 being normal.'''
		   
		return self._vstretch

	def size(self):
		
		'''Returns the fixed size for the gadget, in UI units
		   Values suffixed with 'p' (ie, 20p) are specified in pixels.
		   
		   One UI unit is about 90p.
		   Fixed sized gadgets ignore any stretch attributes applied to them.'''
	
		return tuple([self._width, self._height])

	def width(self):
		
		'''Returns the fixed width for the gadget, in UI units
		
		   Any horizontal stretch is ignored, though the gadget can still stretch verically.'''
		
		return self._width

	def height(self):
		
		'''Returns the fixed height for the gadget, in UI units
		
		   Any vertical stretch is ignored, though the gadget can still stretch horizontally.'''
		   
		return self._height

	def minSize(self):
		
		'''Returns the minimum size of the gadget, in UI units.
		The gadget will never shrink to smaller than this size.'''
		
		return tuple([self._min_width, self._min_height])

	def minWidth(self):
		
		'''Returns the minimum width of the gadget, in UI units.
		The gadget will never shrink to smaller than this size.'''
		
		return self._min_width

	def minHeight (self):
		
		'''Returns the minimum height of the gadget, in UI units.
		The gadget will never shrink to smaller than this size.'''
		
		return self._min_height

	def maxSize(self):
		
		'''Returns maximum size of a gadget, in UI units.'''
		
		return tuple([self._max_width, self._max_height])

	def maxWidth(self):
		
		'''Returns the maximum width of a gadget, in UI units.'''
		
		return self._max_width

	def maxHeight (self):
	
		'''Returns the maximum height of a gadget, in UI units.'''
	
		return self._max_height

	def prefSize(self):
		
		'''Returns the preferred size of the gadget, in UI units.
		The preferred size is the size the gadget would like to be laid out at, if space is available.'''
		
		return tuple([self._pref_width, self._pref_height])	

	def prefWidth(self):
		
		'''Returns the preferred width of the gadget, in UI units.
		The preferred width is the width the gadget would like to be laid out at, if space is available.'''
		
		return self._pref_width

	def prefHeight (self):
		
		'''Returns the preferred height of the gadget, in UI units.
		The preferred height is the height the gadget would like to be laid out at, if space is available.'''
		
		return self._pref_height

	def margin(self):
		
		'''Returns  the margins around the child gadget area in a container, in UI units.'''
		
		return tuple([self._hmargin, self._vmargin])

	def hMargin(self):
		
		'''Returns the horizontal margins around the child gadget area in a container, in UI units.'''
		
		return self._hmargin

	def vMargin(self):
		
		'''Returns  the vertical margins around the child gadget area in a container, in UI units.'''
		
		return self._vmargin

	###def spacing(self):
		
		###'''Returns spacing between children in a container, in UI units.'''
		
		###return self._spacing

	def justify(self):
		
		'''Returns the position the child box within the container,
		when the children in a container cannot grow to fill the entire area of the container.'''
		
		return tuple([self._hjustify, self._vjustify])

	def cell(self):
	  
		'''Returns where in a cell layout this gadget resides.'''
		
		return tuple([self._cell_x, self._cell_y])
	      
	def cellSize(self):
	
		'''Returns how many cells this gadget spans
	
		   1x1 unless specified'''
	
		return tuple([self._cell_width, self._cell_height])
	      
	def cellWidth(self):
		
		'''Returns how many cells wide this gadget spans
		
		  1 unless specified'''

		return self._cell_width
	      
	def cellHeight(self):
		
		'''Returns how many cells this gadget spans vertically
	
		  1 unless specified'''
	   
		return self._cell_height
	      
	def look(self):
	
		'''Returns the look of the gadget'''
	
		return self._look

	def numComponents(self):
	  
		'''Return the number of values stored'''
		
		return self._num_components
	      
	def namingScheme(self):
	  
		'''Return the naming scheme used
		
		   Used only when more than 1 component'''
		   
		return self._naming_scheme
	      
	      
	def scriptCallback(self):
		
		'''Return script as a string that runs on gadget changes.
		
		Note Currently only python callbacks are supported.'''

		# since only a single call back is aloowed
		# (before getting create dialog) present it a string
		result = None
		if len(self._script_callbacks) != 0:
			result  = self._script_callbacks[0]
		  
		return result
      
	def setScriptCallback(self, callback_string):
	  
		''' Set the callback script to provided string.
		This script will run in response to gadget changes.
		
		Note Currently only python callbacks are supported.
		     hscript can be used via hou.hscript()'''

		self._script_callbacks = tuple([str(callback_string)])

		return None


	def clearScriptCallback(self):
	  
		'''Clears callback script from uiScriptGadget Template'''
		
		self._script_callbacks = ()
	  
		return None

## --------------------------------------

	def setName(self, name):
	  
		if isinstance(name, str) == True:
			self._name = name
		else:
			raise hou.TypeError('name must be of type string')
		return None
	
	def setValue(self, val):
		self._value = val
		return None

	def setKeyvalue(self, val):
	  
		'''Normally set on windows, changed whenever a key is pressed and contains the key value.'''
		
		self._keyvalue = val
		return 

	def setStretch(self, h=1,v=1):
		
		'''Makes the gadget stretchable in both directions.
		   Specify floating point values for the amount of stretchiness:
		   0 being none and 1 being normal.
		   
		   All stretchiness values are relative to one another within the same container.
		   A gadget with stretchiness of 2 would stretch twice as much as one of stretchiness 1;
		   the same would occur if they were assigned as 1 and 0.5. '''
		   
		self._hstretch= h 
		self._vstretch= v
		return None

	def setHStretch(self, h=1):
		
		'''Makes the gadget stretchable horizontally.
		   Specify floating point values for the amount of stretchiness:
		   0 being none and 1 being normal.
		
		   Useful for fields and sliders. '''
		   
		self._hstretch= h 
		return None

	def setVStretch(self, v):
		
		'''Makes the gadget stretchable vertically.
		   Specify floating point values for the amount of stretchiness:
		   0 being none and 1 being normal.'''
		
		self._vstretch = v
		return None
	
	def setSize(self, w, h):
		
		'''Specify a fixed size for the gadget, in UI units
		   Pixels can also be used by suffixing the value with 'p' (ie, 20p).
		   
		   One UI unit is about 90p.
		   Fixed sized gadgets ignore any stretch attributes applied to them.'''
		   
		self._width  = w
		self._height = h
		return None

	def setWidth(self, w):
		
		'''Specify a fixed width for the gadget, in UI units.
		
		   Any horizontal stretch is ignored, though the gadget can still stretch verically.'''
		
		self._width = w
		return None

	def setHeight(self, h):
		
		'''Specify a fixed height for the gadget, in UI units.
		
		   Any vertical stretch is ignored, though the gadget can still stretch horizontally.'''
		   
		self._height= h
		return None

	def setMinSize(self, w, h):
		
		'''Set the minimum size of the gadget, in UI units.
		The gadget will never shrink to smaller than this size.'''
		
		self._min_width = w
		self._min_height = h
		return None

	def setMinWidth(self , w):
		
		'''Set the minimum width of the gadget, in UI units.
		The gadget will never shrink to smaller than this size.'''
		
		self._min_width = w
		return None

	def setMinHeight (self, h):
		
		'''Set the minimum height of the gadget, in UI units.
		The gadget will never shrink to smaller than this size.'''
		
		self._min_height = h
		return None

	def setMaxSize(self, w, h):
		
		'''Set the maximum size of a gadget, in UI units.
		
		Generally used in conjunction with STRETCH, to prevent a gadget from getting undesirably large.
		The max size must be larger than both the minimum and preferred size, if specified.'''
		
		self._max_width = w
		self._max_height = h
		return None

	def setMaxWidth(self, w):
		
		'''Set the maximum width of a gadget, in UI units.
		
		Generally used in conjunction with STRETCH, to prevent a gadget from getting undesirably large.
		The max width must be larger than both the minimum and preferred width, if specified.'''
		
		self._max_width = w
		return None

	def setMaxHeight (self, h):
		
		'''Set the maximum height of a gadget, in UI units.
		
		Generally used in conjunction with STRETCH, to prevent a gadget from getting undesirably large.
		The max height must be larger than both the minimum and preferred height, if specified.'''
		
		self._max_height = h
		return None

	def setPrefSize(self, w, h):
		
		'''Set the preferred size of the gadget, in UI units.
		The preferred size is the size the gadget would like to be laid out at, if space is available.
		
		If a min and preferred size are specified for the same dimension, but no stretch is set,
		    the size will still float between the min and preferred size.
		The preferred size must be between the min and max sizes, if specified.'''
		
		self._pref_width = w
		self._pref_height = h
		return None	

	def setPrefWidth(self, w):
		
		'''Set the preferred width of the gadget, in UI units.
		The preferred width is the width the gadget would like to be laid out at, if space is available.
		
		If a min and preferred width are specified, but no stretch is set,
		    the width will still float between the min and preferred width.
		The preferred width must be between the min and max width, if specified.'''
		
		self._pref_width = w
		return None

	def setPrefHeight (self, h):
		
		'''Set the preferred height of the gadget, in UI units.
		The preferred height is the height the gadget would like to be laid out at, if space is available.
		
		If a min and preferred height are specified, but no stretch is set,
		    the height will still float between the min and preferred height.
		The preferred height must be between the min and max heights, if specified.'''
		
		self._pref_height = h
		return None

	def setMargin(self, m): ## should be able to set this same as returned
	
		'''Sets the margins around the child gadget area in a container, in UI units.'''
		
		self._hmargin = m
		self._vmargin = m
		return None

	def setHMargin(self, hm):
		
		'''Sets the horizontal margins around the child gadget area in a container, in UI units.'''

		if any ([
	                 type(hm) == type(int()),
		         type(hm) == type(float())
		        ]):
			store_val = hm ## should this be tuple([hm,hm])

		else: store_val = tuple(hm)
		
		self._hmargin = store_val
		return None


	def setVMargin(self, vm):
		
		'''Sets the vertical margins around the child gadget area in a container, in UI units.'''

		if ([
	             type(vm) == type(int()),
		     type(vm) == type(float())
		    ]):
			store_val = vm

		else: store_val = tuple(vm)
		
		self._vmargin = store_val
		return None


	#def setSpacing(self, hv):
		
		#'''setSpacing(s) or 'setSpacing((h,v))
		
		#Sets the spacing between children in a container, in UI units.
		#The second signature can be used for cell layouts.
		
		#Ignored for overlap layouts.'''
		
		#if ([
	             #type(hv) == type(int()),
		     #type(hv) == type(float())
		    #]):
			#store_val = hv

		#else: store_val = tuple(hv)
		
		#self._spacing = store_val
		#return None


	def setJustify(self, uiScriptJustifyHoriz, uiScriptJustifyVert):
		
		'''Specify where to position the child box within the container,
		when the children in a container cannot grow to fill the entire area of the container.'''
		
		h = uiScriptJustifyHoriz
		v = uiScriptJustifyVert
		self._hjustify = h
		self._vjustify = v
		return None
	      
	def setCell(self, x, y):
	  
		'''Specify where in a cell layout this gadget resides.'''
		
		self._cell_x = x
		self._cell_y = y
		return None
	      
	def setCellSize(self, w, h):
	
		'''Specify how many cells this gadget spans
	
		   1x1 unless specified'''
	
		self._cell_width  = w
		self._cell_height = h
		return None
	      
	def setCellWidth(self, w):
		
		'''Specify how many cells wide this gadget spans
		
		   1 unless specified'''

		self._cell_width = w
		return None
	      
	def setCellHeight(self, h):
		
		'''Specify how many cells this gadget spans vertically
	
	        1 unless specified'''
	   
		self._cell_height = h
		return None      

	def setLook(self, uiScriptLook):
		
		'''Specifies the look of the gadget, though this is mostly used for container feels and windows'''
		
		look = uiScriptLook
		self._look = look
		return None

	def setNumComponents(self, num_components):
	  
		'''Specify the number of values stored'''
		
		self._num_components = int(num_components)
		return None
	      
	def setNamingScheme(self, uiScriptNamingScheme):
	  
		'''Specify the naming scheme used
		
		   No effect when only 1 component'''
		   
		naming_scheme = uiScriptNamingScheme
		self._naming_scheme = naming_scheme
		   
		return None

## --------------------------------------

	def _printOkay(self, value):

		if any([
		value == None,
		value == (None, None),
		value == '',
		value == ('',''),
		value == ()
		]):
			result = False
		else:
			result = True

		return result


	def _printKeyvalue(self):

		if  self._printOkay(self._keyvalue):
			result = 'KEYVALUE(' + str(self._keyvalue) + ')'
		else: result = None

		return result


	#def _printLayout(self): ## how to deal with this if layout is brough to the container

		#if  self._printOkay(self._layout):
			#result = 'LAYOUT(' + self._layout + ')'
		#else : result = None

		#return result
		

	def _printStretch(self):
	  
		if self._hstretch == self._vstretch == 1:
			result = 'STRETCH'
	  
	  	elif all([
			self._printOkay(self._vstretch),
			self._printOkay(self._hstretch)
		]):      
			result = 'STRETCH(' + str(self._hstretch) +','+ str(self._vstretch) +')'

		elif self._printOkay(self._vstretch):
			result = 'VSTRETCH(' + str(self._vstretch) +')'

		elif self._printOkay(self._hstretch):
			result = 'HSTRETCH(' + str(self._hstretch) +')'
		
		else: result = None

		return result


	# size printed in ui unit?
	def _printSize(self):

		if all([
			self._printOkay(self._width),
			self._printOkay(self._height)
		]):      
			result = 'SIZE(' + str(self._width) +','+ str(self._height) +')'

		elif self._printOkay(self._width):
			result = 'WIDTH(' + str(self._width) +')'

		elif self._printOkay(self._height):
			result = 'HEIGHT(' + str(self._height) +')'
		
		else: result = None

		return result


	def _printMinSize(self):

		if all([
			self._printOkay(self._min_width),
			self._printOkay(self._min_height)
		]):      
			result = 'MIN_SIZE(' + str(self._min_width) +','+ str(self._min_height) +')'

		elif self._printOkay(self._min_width):
			result = 'MIN_WIDTH(' + str(self._min_width) +')'

		elif self._printOkay(self._min_height):
			result = 'MIN_HEIGHT(' + str(self._min_height) +')'
		
		else: result = None

		return result


	def _printMaxSize(self):

		if all([
			self._printOkay(self._max_width),
			self._printOkay(self._max_height)
		]):      
			result = 'MAX_SIZE(' + str(self._max_width) +','+ str(self._max_height) +')'

		elif self._printOkay(self._max_width):
			result = 'MAX_WIDTH(' + str(self._max_width) +')'

		elif self._printOkay(self._max_height):
			result = 'MAX_HEIGHT(' + str(self._max_height) +')'
		
		else: result = None

		return result


	def _printPrefSize(self):

		if all([
			self._printOkay(self._pref_width),
			self._printOkay(self._pref_height)
		]):      
			result = 'PREF_SIZE(' + str(self._pref_width) +','+ str(self._pref_height) +')'

		elif self._printOkay(self._pref_width):
			result = 'PREF_WIDTH(' + str(self._pref_width) +')'

		elif self._printOkay(self._pref_height):
			result = 'PREF_HEIGHT(' + str(self._pref_height) +')'
		
		else: result = None

		return result


	def _printHMargin(self):

		if self._printOkay(self._hmargin):

			if type(self._hmargin) == type(()):
				result = 'HMARGIN' +str(self._hmargin)
			else: result = 'HMARGIN(' + str(self._hmargin) + ')'
		
		else: result = None
		
		return result


	def _printVMargin(self):

		if self._printOkay(self._vmargin):

			if type(self._vmargin) == type(()):
				result = 'VMARGIN' +str(self._vmargin)
			else: result = 'VMARGIN(' + str(self._vmargin) + ')'
		
		else : result = None

		return result


	#def _printSpacing(self):

		#if not self. _printOkay(self._spacing):
			#return None

		#if self._layout == uiScriptLayout.cell:
			#if type(self._spacing) == type(()):
				#result = 'SPACING' + str(self._spacing)
			#else: result = 'SPACING(' +str(self._spacing)+ ',' + str(self._spacing) +')'
		#else:
			#if type(self._spacing) == type(()):
				#result = 'SPACING(' + str(self._spacing[0]) +')'
			#else: result = 'SPACING('+ str(self._spacing)+')'
			
		#return result


	def _printJustify(self):

		if all([
			self._printOkay(self._hjustify),
			self._printOkay(self._vjustify)
			]):
			result = 'JUSTIFY(' + self._hjustify.name() +','+ str(self._vjustify.name()) +')'
		else:
			result = None

		return result
	      
	def _printCell(self):
	  
		if all([
			self._printOkay(self._cell_x),
			self._printOkay(self._cell_y)
			]):
		  
			result = 'CELL(' + str(self._cell_x) +','+ str(self._cell_y) +')'
		else: result = None

		return result
	      

	def _printCellSize(self):

		if all([
			self._printOkay(self._cell_width),
			self._printOkay(self._cell_height)
		]):      
			result = 'CELLSIZE(' + str(self._cell_width) +','+ str(self._cell_height) +')'

		elif self._printOkay(self._cell_width):
			result = 'CELLSIZE(' + str(self._cell_width) +',1)'

		elif self._printOkay(self._cell_height):
			result = 'CELLSIZE(1,' + str(self._cell_height) +')'
		
		else: result = None

		return result
	      

	def _printLook(self):
		
		if self._printOkay(self._look):
			result = 'LOOK(' + self._look.name() +')'
		else: result = None

		return result


	def _printGadgetAttributes(self):
		return list()


	def _printCmds(self): 
		
		cmds = []

		if self._disable:
			cmds.append('DISABLE ' + self._name + '.val')

		if self._export:
			cmds.append('EXPORT ' + self._name + '.val')

		### IMPLEMENT OPEN COMMAND HERE ---->>> needs window name for ref!!!

		result = ';\n'.join(cmds)
		return result


	def _printBaseAttribute(self):
		
		base_attribs = [
				self._printKeyvalue(),
				#self._printLayout(),
				self._printStretch(),
				self._printSize(),
				self._printMinSize(),
				self._printMaxSize(),
				self._printPrefSize(),
				self._printHMargin(),
				self._printVMargin(),
				#self._printSpacing(),
				self._printJustify(),
				self._printCell(),
				self._printCellSize(),
				self._printLook()
			]
		result = [i for i in base_attribs if i !=None]
		return result


	def _printAllAttributes(self):

		base_attribs = self._printBaseAttribute()
		gadget_attribs = self. _printGadgetAttributes() 
		all_attribs = gadget_attribs + base_attribs
		
		result = ' '.join(all_attribs)
		
		return result
		
