# ==============================================================================
#           basic containers class used in uiScriptPy module
#  ----------------------------------------------------------------------------
#  Establishes the basic uiScriptContainer
#  Combine uiScript ROW, COL, and COLLAPSER
# ==============================================================================

from   enums        import *
import utils
from   base_classes import *
from   container_class import*


class BasicUiScriptContainer(UiScriptContainer):

	def __init__(self, name, uiScriptGadgets=(), basic_container_type=uiScriptBasicContainerType.row):

		self.setName(name)
		
		self._children             = uiScriptGadgets
		self._basic_container_type = basic_container_type
		self._container_type       = self._basic_container_type
		self._value                = 1
		
		return None
		
	# for declaration
	_children             = ()
	_basic_container_type = None
	_container_type       = None
	_type                 = uiScriptGadgetType.container


	# specific to this gadget
	_label        = None
	_label_width  = None
	
	# specific gadget attibutes

	# ---------------------------------
	

	def label(self):

		'''Return label diplayed as a string.'''
		return self._label

	# ---------------------------------
	

	def setLabel(self, label):
	  
		'''Set string for label diplayed '''
		self._label = label
		return None
		
	# ---------------------------------
	
	def _printDeclarations(self):
	# prints the container declaration with refs to children
	
		# value declaration
		names = utils._gadgetTemplateNamesTuple(
		                                        self.name(),
		                                        self.namingScheme(),
		                                        self.numComponents()
		                                        )
		val_decl = names[0] + ':= ' +  str(self._value) +';'
		
		# gadget declaration ...
		gadget_decl = self._name + '.gad ='
		
		container_type = self._container_type.name().upper()
		gadget_decl +=  ' ' + container_type
		
		# set the label on collapsers
		if self._container_type == uiScriptBasicContainerType.collapser:

			if self._printOkay(self._label) == False:
				self.setLabel(self._name)
			  
			gadget_decl +=  ' "'+ self._label + '"'

		# value atrrib goes on a seperate line
		val_attrib  = 'VALUE(' + self._name + '.val)\n'	

		all_attribs = self._printAllAttributes()
		all_attribs = '    ' + val_attrib + '    ' +all_attribs
		
		gadget_refs = [x._name + '.gad;\n' for x in self._children]
		gadget_refs_string = '    ' + ('    '.join(gadget_refs) )
		
		gadget_decl += '\n{\n' + all_attribs + '\n\n' + gadget_refs_string + '}'

		# commands
		cmds        = self._printCmds()

		# resulting string
		result = val_decl + '\n' + gadget_decl +'\n'+cmds
		
		return result

