# ==============================================================================
#                buttons used in uiScriptPy module
#  ----------------------------------------------------------------------------
#  Various Button Uiscript Feel via a button object.
#      TOGGLE_BUTTON  TOGGLE_ICONBUTTON  ACTION_BUTTON  ACTION_ICONBUTTON  
# ==============================================================================


from enums        import *
from base_classes import *

import utils


class ButtonUiScriptGadget(UiScriptGadget):

	def __init__(self, name, label=None, icon=None, toggle=False):

		self.setName(name)
		self.setLabel(label)
		
		self._value   = 0
		self._type    = uiScriptGadgetType.button
		
		# label is opt, but as convention...
		if not label: self._label = self._name
		
		# store the icon if provided
		if icon:
			self._icon = icon
			
		# change to toggle if requested
		if toggle:
			self._button_type = uiScriptButtonType.toggle
			
		# to be consistant default limit width to label
		self.setHStretch(0)

		return None


	#def __repr__(self):
		#pass

	# for declaration
	_type        = uiScriptGadgetType.button
	_button_type = uiScriptButtonType.action

	# specific to this gadget
	_label           = None
	_label_width     = None ## check if this works
	_icon            = None
	
	_shared_variable = None

	# specific gadget attibutes
	
	# enums for 'look' already + applies icons only- for attrib LOOKFLAT
	_icon_flat       = False 
	# scale enum controls size, other places 'size' is ui units
	_icon_scale      = uiScriptIconButtonSize.small 
	_sends           = None
	
	# access
	def buttonType(self):
	  
	  	'''Enumeration of this button type.'''
		return self._button_type


	def icon(self):
	
		'''Return the icon name if in use.''' 
		return self._icon
		
	def sharedVariable(self):
		return self._shared_variable

	def label(self):
	
		'''Return label diplayed as a string.
		
		Note. Has no effect when using an Icon'''
		return self._label

	def labelWidth(self):
	  
		'''Return the width of label displayed'''
		return self._label_width
		
	def isToggle(self):
	  
		'''Return if this button is a toggle.
		
		A toggle button switches between 0 and 1.
		1 is selected'''
	  
		if self._button_type == uiScriptButtonType.toggle:
			result = True
		else:
			result = False
		return result

	def isIconFlat(self):
	
		'''Return if an icon appears or not.
		Flat buttons have no shadow around icon button,
		    so they appear "inline" with the background.
		    
		Note. only affects Buttons using icons'''
		return self._icon_flat
		
	def iconScale(self):
	
		'''Return the size of an icon button to an enum
		    of standard sizes.
	  
		Note. only affects Buttons using icons'''
		return self._icon_scale
		
	def sends(self):
		return self._sends
		
	# ---------------------------------
	
		
	def setIcon(self, iconname):
	  
		'''Set the name icon file to use.''' 
		
		self._icon = iconname
		return None

		
	def setSharedVariable(self, var_name):
		self._shared_variable = str(var_name)
		return None
		
		
	def setLabel(self, label):
	  
		'''Set string for label diplayed '''

		self._label = label
		return None

		
	def setLabelWidth(self, label_width):
	  
		'''Set the width of label displayed'''
	
		self._label_width = float(label_width)
		return None
		

	def setIsToggle(self, is_toggle_button=True):
	  
		'''Set if this button is a toggle or not.
		
		A toggle button switches between 0 and 1.
		1 is selected'''
	
		if bool(is_toggle_button) == is_toggle_button:
			if bool(is_toggle_button) == True:
				self._button_type = uiScriptButtonType.toggle
			else:
				self._button_type = uiScriptButtonType.action
		return None

		
	def setIsIconFlat(self, is_icon_flat=True):
	  
		'''Set if an icon button is flat or not.
		    Flat buttons have no shadow around icon button,
		    so they appear "inline" with the background.
		    
		Note. only affects Buttons using icons'''
	  
		if bool(is_icon_flat) == is_icon_flat:
			self._icon_flat = bool(is_icon_flat)  
		return None
	

	def setIconScale(self, uiScriptIconButtonSize): 
	  
		'''Set the size of an icon button to an enum
		    of standard sizes.
	  
		Note. only affects Buttons using icons'''

		self._icon_scale = uiScriptIconButtonSize
		return None


	def setSends(self, int_val):
		'''
		'''
		if any([
		        type(int_val) == type(float()),
		        type(int_val) == type(int())
		       ]):
			 self._sends = int(int_val)
		return None


	# ---------------------------------

	def _printDeclarations(self):

		# value declaration
		if not self._printOkay(self._shared_variable):
			#val_decl = self._name + '.val:= ' + str(self._value) + ';'
			names = utils._gadgetTemplateNamesTuple(
		                                        self.name(),
		                                        self.namingScheme(),
		                                        self.numComponents()
		                                        )
			val_decl = names[0] + ':= ' +  str(self._value) +';'
		else:
			val_decl = ''

		# gadget declaration ...
		gadget_decl = self._name + '.gad ='
		
		# ... use the enum to dtermine the gadget...
		if self._button_type == uiScriptButtonType.toggle:
		  
			button_type = 'TOGGLE'
		else:
			button_type  = 'ACTION'
			
		gadget_decl +=  ' ' + button_type
		    
		if self._printOkay(self._icon):
		  
			gadget_decl += '_ICONBUTTON'
			gadget_decl +=  ' "'+ self._icon + '"'
			
		else:  
			gadget_decl += '_BUTTON'
			gadget_decl +=  ' "'+ self._label + '"'

			
		if self._printOkay(self._label_width):
			gadget_decl += ':' + str(self._label_width)

		# use a shared variable for the value name or the norm
		if self._printOkay(self._shared_variable):
			value_name =  self._shared_variable + '.val'
		else:
			value_name =  self._name + '.val'
		  
		gadget_decl += ' VALUE(' + value_name + ')'	

		all_attribs = self._printAllAttributes()
		cmds        = self._printCmds()

		result = val_decl +'\n'+ gadget_decl +' '+all_attribs+';\n'+cmds
		
		return result


	def _printSend(self):

		if self._printOkay(self._sends):
			result = 'SENDS(' + str(self._sends) + ')'
		else : result = None

		return result
		
		
	def _printIsIconFlat(self):
	  
		if all([
		        self._printOkay(self._icon),
		        self._icon_flat == True
		       ]):
			 result = 'LOOKFLAT'
		else:
			result = None
			
		return result		 
		

	def _printIconScale(self):
	  
		if all([
		        self._printOkay(self._icon_scale),
		        self._printOkay(self._icon)
		      ]):
			icon_size = self._icon_scale.name().upper()
			result = 'UI_FLAT_ICON_BTN_SIZE_' + icon_size
		else:
			result = None
			
		return result
		

	def _printGadgetAttributes(self):

		gadget_attribs = [
			          self._printSend(),
			          self._printIsIconFlat(),
			          self._printIconScale()
			         ]
		result = [i for i in gadget_attribs if i !=None]
		return result
