# ==============================================================================
#                      color widget used in uiScriptPy Lib
#  ----------------------------------------------------------------------------
#  file fields and related combines UI script ...
#      COLOR_FIELD COLOR_ALPHA_FIELD 
# ==============================================================================

from enums        import *
import utils
from base_classes import *


class ColorUiScriptGadget(UiScriptGadget):

	def __init__(self, name, label, has_alpha=False): 

		self.setName(name)
		self.setLabel(label)
		self.setIsColorAlpha(has_alpha)
		
		self._value   = (1,1,1,1)

		if label == None: self.setLabel(self.name())
		
		self.setNamingScheme(uiScriptNamingScheme.RGBA)
		
		return None

	#def __repr__(self):
		#pass

	# for declaration
	_type          = uiScriptGadgetType.color
	_color_type    = uiScriptColorType.color

	# specific to this gadget
	_label           = None
	_label_width     = None 

	# specific gadget attibutes

	
	# ---------------------------------

	# access
	
	def label(self):
		return self._label

		
	def labelWidth(self):
		return self._label_width

		
	def isColorAlpha(self):
	  
		if self._color_type == uiScriptColorType.color_alpha:
			result = True
		else:
			result = False
			
		return result
		
	# ---------------------------------
	
	
	def setLabel(self, label):
	  
		self._label = label
		return None

		
	def setLabelWidth(self, label_width):
	  
		if label_width == None:
		  
			self._label_width = None	
		else :
			self._label_width = float(label_width)
			
		return None
		
	      
	def setIsColorAlpha(self, true_or_false=True):
	  
		if bool(true_or_false) == True:
			self._color_type = uiScriptColorType.color_alpha
			self._num_components = 4
			
		elif bool(true_or_false) == False:
			self._color_type = uiScriptColorType.color
			self._num_components = 3
			
		return None

	# ---------------------------------

	def _printDeclarations(self):

		# value declaration 
		if self._printOkay(self._value):
			value = self._value
		else: value = ()
		
		# drop or add color.v3.val if alpha is set ...
		values = list(value)
		
		# ... if set to None will ensure all white solid
		values.extend([1.0,1.0,1.0,1.0])
		
		# how big are we talking
		if self._color_type == uiScriptColorType.color_alpha:
			values = values[0:4]
		else: values = values[0:3]
		
		# a value is declared per vector element
		val_decl = ''
		
		names = utils._gadgetTemplateNamesTuple(
		                                        self.name(),
		                                        self.namingScheme(),
		                                        self.numComponents()
		                                        )
		for i in range(len(values)):
			#val_decl += self._name + '.v' + str(i) + '.val := ' + str(float(values[i])) + ';\n'
			val_decl += names[i] + ':= ' + str(float(values[i])) + ';\n'

		# gadget declaration
		gadget_decl = self._name + '.gad ='
		
		# use color alpha declaration id alpha is toggled on
		color_type = ' ' + (self._color_type.name().rsplit('.', 1)[-1]).upper() + '_FIELD'

		gadget_decl += color_type

		# add the label if one has bween set
		if self._printOkay(self._label):
			gadget_decl += ' "'+self._label +'"'
			
			if self._printOkay(self._label_width):
				gadget_decl += ':' + str(self._label_width)

		# value

		#val_names_list = [self._name + '.v' + str(i) + '.val' for i in range(len(values))]
		val_arg_str    = str( ', '.join(names) )
		
		gadget_decl += ' VALUE(' + val_arg_str + ')'	

		all_attribs = self._printAllAttributes()
		cmds        = self._printCmds()

		result = val_decl +'\n'+ gadget_decl +' '+all_attribs+';\n'+cmds

		return result

	

	def _printGadgetAttributes(self):

		gadget_attribs = [
			          #,
			          #,
			          #
			         ]
		result = [i for i in gadget_attribs if i !=None]
		return result
