# ==============================================================================
#           base container class used in uiScriptPy module
#  ----------------------------------------------------------------------------
#  Establishes the uiScriptGadget and uiScript container classes
#       base classes for all gadgets including  containers windows etc...
# ==============================================================================

from   enums        import *
import utils
from   base_classes import *

class UiScriptContainer(UiScriptGadget):
	
	
	def __init__(self, name, uiScriptGadgets=(), container_type=uiScriptContainerType.row):

		self._name           = name
		self._children       = uiScriptGadgets
		self._container_type = container_type
		
		return None
	
	# storage
	_children       = ()
	_container_type = None
	_type           = uiScriptGadgetType.container
	
	# attributes
	_layout         = None
	_spacing        = (0.05, 0.05)
	


	# ---------------------------------


	def containerType(self):
	  
		'''Return Container Type'''
		
		return self._container_type	  


	def uiScriptGadgets(self):
		
		'''Return a tuple of UI Script Gadgets stored inside this container.'''
		
		return self._children
		
	def layout(self):
	  
		''' Returns value from enumeration uiScriptLayout '''

		return self._layout
	      
	      
	def spacing(self):
		
		'''Returns spacing between children in a container, in UI units.'''
		
		return self._spacing
		   
	# ---------------------------------


	def addUiScriptGadget(self, UiScriptGadget):
		
		'''Append a UI Script Gadget to the end of the list of gadgets inside this container'''
		
		if  utils._isGadget(UiScriptGadget) == True:
		  
			children = list(self._children)
			children.append(UiScriptGadget)
			self._children = tuple(children)
		
		return None


	#def setContainerType(self, uiScriptContainerType): # this cant be setable when inhereted
	
		#'''Set the type of container to a UIScriptGadget enum value.'''

		#if uiScriptContainerType in dir(uiScriptContainerType):
			#self._container_type = uiScriptContainerType
		
		#return None


	def setUiScriptGadgets(self, UiScriptGadgets):
		
		'''Replace the UiScriptGadgets inside this container with a new sequence of UiScriptGadgets.'''
		
		# check all passed are gadgets
		for gadget in list(UiScriptGadgets):
			if utils._isGadget(gadget) == False:
				return None

		# make a tuple and store
		self._children = tuple(UiScriptGadgets)

		return None
		
		
	def setLayout(self, uiScriptLayout):
	  
		'''Sets a containers layout style.
		   from enumeration uiScriptLayout '''

		self._layout = uiScriptLayout
		return None

      
	def setSpacing(self, hv):
		
		'''setSpacing(s) or 'setSpacing((h,v))
		
		Sets the spacing between children in a container, in UI units.
		The second signature can be used for cell layouts.
		
		Ignored for overlap layouts.'''
		
		if ([
	             type(hv) == type(int()),
		     type(hv) == type(float())
		    ]):
			store_val = hv

		else: store_val = tuple(hv)
		
		self._spacing = store_val
		return None
		   
	# ---------------------------------
	
	#def _printChildNames(self):
		#pass
	
	def _printDeclarations(self):
	# prints the container declaration with refs to children
		return None
	     

	def _printLayout(self):

		if  self._printOkay(self._layout):
			result = 'LAYOUT(' + self._layout.name() + ')'
		else : result = None

		return result


	def _printSpacing(self):

		if not self. _printOkay(self._spacing):
			return None

		if self._layout == uiScriptLayout.cell:
			if type(self._spacing) == type(()):
				result = 'SPACING' + str(self._spacing)
			else: result = 'SPACING(' +str(self._spacing)+ ',' + str(self._spacing) +')'
		else:
			if type(self._spacing) == type(()):
				result = 'SPACING(' + str(self._spacing[0]) +')'
			else: result = 'SPACING('+ str(self._spacing)+')'
			
		return result


	def _printGadgetAttributes(self):

		gadget_attribs = [
			          self._printLayout(),
			          self._printSpacing(),
			          #self._printSliderRangeValue()
			         ]
		result = [i for i in gadget_attribs if i !=None]
		return result
	 

	def _printContainerContents(self, level=0, max_levels=0):
	# gatehrs children and call their print methods
	# recursively does the same for levels arg.
	# levels=0 recurses all levels

	
		## get out if level >= max_levels
		#if level >= max_levels:
			#return None

		result   =''
		tab      = '    '
		indent   = tab*level
		level    = level+1
		
		for child in self._children:
			
			# child is a container
			if utils._isContainer(child):
				# recurse children
				result = result +'\n\n' + child._printContainerContents(level, max_levels)
				
			# print declarations for all gadgets
			result = result + '\n\n' + child._printDeclarations()

		
		result = result.replace('\n', '\n'+tab)
		
		return result
		


	def _printContainer(self):

		container_str = self._printContainerContents()
		container_str += '\n\n' 
		container_str += self._printDeclarations()
		
		return container_str
  
	# inner most val/gadget declarion -- maybr seperate procedure

	# print declaration
	# print {
	# print basse attribs
	# print subconatainer if any (repeat)
		# print { (repeat)
	# print content gadgets
	# print closing }'s
	# print cmds after each }