# ==============================================================================
#        extention methods for hou.Dialog & function to put them there
#  ----------------------------------------------------------------------------
#  Adds methods to manibulate dialog visability, buiding and destruction
#       functionality for returning and setting Gadgets
# ==============================================================================

import types
import hou

import utils
from ui_gadget import *


def _extendHouDialog(uiScriptWindow, hou_dialog):
 
	# need a places to stash uiscript template`
	hou_dialog._ui_script_dialog = uiScriptWindow
 
	# make the uiScriptPy object retievable
	hou_dialog.uispy_uiScriptDialog = types.MethodType(_uiScriptDialog, hou_dialog)

	# add conveiniances for visibility
	hou_dialog.uispy_show = types.MethodType(_dialogShow, hou_dialog)
	hou_dialog.uispy_hide = types.MethodType(_dialogHide, hou_dialog)
	hou_dialog.uispy_isVisbile = types.MethodType(_dialogIsVisbile, hou_dialog)
	hou_dialog.uispy_setIsVisbile = types.MethodType(_dialogSetIsVisible, hou_dialog)

	# .. or destroy it a little cleaner
	hou_dialog.uispy_destroy = types.MethodType(_dialogDestroy, hou_dialog)

	# add gadget retrival methods
	hou_dialog.uispy_gadgetTuple  = types.MethodType(_dialogGadgetTuple, hou_dialog)
	hou_dialog.uispy_gadgetTuples = types.MethodType(_dialogGadgetTuples, hou_dialog)

	hou_dialog.uispy_gadget = types.MethodType(_dialogGadget, hou_dialog)
	hou_dialog.uispy_gadgets = types.MethodType(_dialogGadgets, hou_dialog)
	
	# an awkward gadget setting method for convieniance
	hou_dialog.uispy_setGadgets = types.MethodType(_dialogSetGadgets, hou_dialog)

	# attatch callbacks
	hou_dialog = _processCallbacks(hou_dialog)

	# give it back...
	return hou_dialog
      
      
def _dialogShow(hou_dialog):
  
	'''Display a hidden dialog
	
	can hide by uispy_hide() or uispy_setIsVisible(0)'''

	uiScriptWindow = hou_dialog._ui_script_dialog

	var_name = uiScriptWindow.name() + '.val'
	hou_dialog.setValue(var_name, 1)
	return None


def _dialogHide(hou_dialog):
  
	'''Hide this dialog.
	
	    can unhide by uispy_show() or uispy_setIsVisible(1)'''

	uiScriptWindow = hou_dialog._ui_script_dialog
	
	var_name = uiScriptWindow.name() + '.val'
	hou_dialog.setValue(var_name, 0)
	return None
      
def _dialogDestroy(hou_dialog, hide=True):
  
	'''Destroy this dialog.
	
	    hide=True hide dialog before destroying.
	    When false the object will remain shown although inaccessible'''
	    
	uiScriptWindow = hou_dialog._ui_script_dialog
	
	if hide == True:
		var_name = uiScriptWindow.name() + '.val'
		hou_dialog.setValue(var_name, 0)
		
	hou_dialog.destroy()
	    
	return None
	    
def _dialogSetIsVisible( hou_dialog, true_or_false):
  
	'''Dialog visibility.
	
	    see also uispy_show() or uispy_hide()'''
	    
	uiScriptWindow = hou_dialog._ui_script_dialog    

	visibility = bool(true_or_false)
	var_name = uiScriptWindow.name() + '.val'
		
	hou_dialog.setValue(var_name, visibility)
	return None


def _dialogIsVisbile(hou_dialog):
  
	'''Retruns dialog visibility'''
	
	ui_script_win = hou_dialog._ui_script_dialog    
	var_name = ui_script_win.name() + '.val'
	
	return bool(hou_dialog.value(var_name))



def _uiScriptDialog(hou_dialog):
	
	''''Returns the uiScriptPy Object that created Dialog'''
	
	return hou_dialog._ui_script_dialog


#============== not implemented ==============

#?? dialogs do not have network children so no path?? only name??
#switchers are like stick notes and do not count in the path..

def _setUiScriptDialog(hou_dialog, DialogUiScriptWindow):

	return None

def _dialogRebuild(hou_dialog):
  
	'''Destroys and recreates dialog'''
	
	return None
      
  
def _dialogGadgets(hou_dialog):
  
	'''Return a list of the Gadget's on this dialog/window.'''
  
	ui_script_win = hou_dialog.uispy_uiScriptDialog()
	all_gadget_tmplts = ui_script_win._flatGadgetsByType()
	
	# get a list of basic gadgets templates only ...
	gadget_tmplt_dicts = list(all_gadget_tmplts['gadgets'])
	
	# .... don't care about the path in this case
	gadgets_tmplts = [ x['gadget'] for x in  gadget_tmplt_dicts ]
	
	# build a list of gadget for user manip
	gadgets = []

	# create a gadget per componant
	for gadget_tmplt in gadgets_tmplts:
	  
		for comp_index in range(gadget_tmplt.numComponents()):
			gadget= Gadget(
			               hou_dialog=hou_dialog,
			               ui_script_gadget=gadget_tmplt,
			               comp_index=comp_index
		                )

			gadgets.append(gadget)
		
	# results
	results = gadgets
	return results


def _dialogGadgetTuples(hou_dialog):
  
	'''Return a list of the GadgetTuple's on this dialog/window.'''
  
	ui_script_win = hou_dialog.uispy_uiScriptDialog()
	all_gadget_tmplts = ui_script_win._flatGadgetsByType()
	
	# get a list of basic gadgets templates only ...
	gadget_tmplt_dicts = list(all_gadget_tmplts['gadgets'])
	
	# .... don't care about the path in this case
	gadgets_tmplts = [ x['gadget'] for x in  gadget_tmplt_dicts ]
	
	# build a list of gadget for user manip
	gadget_tuples = []

	for gadgets_tmplt in gadgets_tmplts:
			
		gadget_tuple= GadgetTuple(
		                          hou_dialog=hou_dialog,
		                          ui_script_gadget=gadgets_tmplt
		                         )
		
		gadget_tuples.append(gadget_tuple)

	# results
	results = gadget_tuples
	return results


def _dialogGadget(hou_dialog, name):

	'''Returns the named Gadget or None'''
	
	ui_script_win = hou_dialog.uispy_uiScriptDialog()
	all_gadget_tmplts = ui_script_win._flatGadgetsByType()
	
	# get a list of basic gadgets templates only ...
	gadget_tmplt_dicts = list(all_gadget_tmplts['gadgets'] +
	                          all_gadget_tmplts['containers'])
	
	# .... don't care about the path in this case
	gadgets_tmplts = [ x['gadget'] for x in  gadget_tmplt_dicts ]

	# default match is None
	gadget = None

	# loop over uiscript templates 
	for gadget_tmplt in gadgets_tmplts:
	  
		# pull potential Gadget() names and match
		gadget_size  = gadget_tmplt.numComponents()
		scheme       = gadget_tmplt.namingScheme()
		tmplt_name   = gadget_tmplt.name()
		gadget_names = utils._gadgetNamesTuple(
		                                       name=tmplt_name,
		                                       uiScriptNamingScheme=scheme,
		                                       gadget_size=gadget_size)
		gadget_names = list(gadget_names)

		# go through the companant names ...
		for comp_index in range(gadget_size):
		
			# ... and get out when a match is found ...
			if gadget_names[comp_index] == name:

				# ... create a gadget.
				gadget= Gadget(
				               hou_dialog=hou_dialog,
				               ui_script_gadget=gadget_tmplt,
				               comp_index=comp_index
				              )
				# the quest is over ...
				# stopping checking componants
				break
	      
		# the quest is over ...
		# stopping checking templates
		if gadget != None:
			break
	
	return gadget


def _dialogGadgetTuple(hou_dialog, name):
  
	'''Returns the named GadgetTuple or None'''
	
	#  just find the template and buid Tuple
	ui_script_win = hou_dialog.uispy_uiScriptDialog()
	ui_script_gadget = ui_script_win.find(name)
	
	if ui_script_gadget == None:
		gadgetTuple = None
	else:
		gadgetTuple= GadgetTuple(
		                         hou_dialog=hou_dialog,
		                         ui_script_gadget=ui_script_gadget
		                        )
	return gadgetTuple

  
	return None


def _dialogSetGadgets(hou_dialog, gadget_dict):
  
	'''Set multiple Gadget via a dictionary gadget_name:value'''
	
	names = gadget_dict.keys()
	for name in  names:
	  
		gadget = _dialogGadget(hou_dialog, name)
		if gadget != None:
			gadget.set(gadget_dict[name])
		
	return None


def _dialogGadgetsInFolder():
	return None
  
def _dialogGadgetTuplesInFolder():
	return None

# this is for processing an attatching non-instance callbacks
def _processCallbacks(hou_dialog):

	ui_script_win = hou_dialog.uispy_uiScriptDialog()
	all_gadget_tmplts = ui_script_win._flatGadgetsByType()
	
	# get a list of basic gadgets templates only ...
	gadget_tmplt_dicts = list(all_gadget_tmplts['gadgets'] +
	                          all_gadget_tmplts['containers'])
	
	# .... don't care about the path in this case
	gadget_tmplts = [ x['gadget'] for x in  gadget_tmplt_dicts ]
  
	# iterate the ui script gadget template adding callbacks found
	for gadget_tmplt in gadget_tmplts:
	  
		# store callback or skip template when None
		callback     = gadget_tmplt.scriptCallback()
		if callback == None:
			continue
		else:
			callback = str(callback)
	  
		# ui_script_names needs to be munged via utils lib
		gadget_size  = gadget_tmplt.numComponents()
		scheme       = gadget_tmplt.namingScheme()
		tmplt_name   = gadget_tmplt.name()
		
		ui_script_names = utils._gadgetTemplateNamesTuple(
		  
		                                       name=tmplt_name,
		                                       uiScriptNamingScheme=scheme,
		                                       gadget_size=gadget_size
		                                       )
		# add callback per companant per
		# will need to revisit his to support multi lang callbacks
		for ui_script_name in ui_script_names:
		
			filename = ui_script_name.replace('.', '_')
			hou_dialog.addCallback(
			  
			           ui_script_name,
			           lambda: eval(compile(callback,filename,'exec'))
		                   )
	return hou_dialog


#ascode  ## UGHHHH
