# ==============================================================================
#                Label used in uiScriptPy Lib
#  ----------------------------------------------------------------------------
#  Various label fields and related combines US script ...
#      LABEL SQUISHY_LABEL
# ==============================================================================

# interesting combo o lable with and squishy
# labelwidth does NOT work with dynamic labels

from enums        import *
from base_classes import *

import utils


class LabelUiScriptGadget(UiScriptGadget):

	def __init__(self, name, label=None): 

		self.setName(name)
		self.setLabel(label)

		# default to name when not explicitly provided
		if label == None:
			self._label = self._name
		
		# with a dynamic lable the _val and _label are the same
		self._value = self._label

		return None


	#def __repr__(self):
		#pass

	# for declaration
	_type        = uiScriptGadgetType.label
	_label_type  = uiScriptLabelType.label

	# specific to this gadget
	_label           = None
	_label_width     = None 

	_squish_max     = None
	_text_look      = None

	# specific gadget attibutes
	#_sends        = None
	
	# ---------------------------------

	# access
	
	def label(self):

		'''Return label diplayed as a string'''
		
		return self._label

		
	#def labelWidth(self):
		#return self._label_width


	def isSquishy(self):
	  
		'''Return if the label is squishy or not
		
		       Squishy labels shrink smaller than full label size,
		       to show max chars (default 4) when space unavailable.
		   
		       Useful when user-defined/data-defined,
		       to prevent excessive wide labels.
		   
		   See also squichMax()'''
	  
		if self._label_type == uiScriptLabelType.squishy_label:
			result = True
		else: result = False
		
		return result


	def squishMax(self):
	  
		'''Return the max chars displayed for a squishy label
		
		       Squishy labels shrink smaller than full label size,
		       to show max chars (default 4) when space unavailable.
		   
		       Useful when user-defined/data-defined,
		       to prevent excessive wide labels.
		   
		   See also isSquishy()'''
		   
		return self._squish_max


	def textLook(self):
	  
		'''Returns a enumeration for the look of the text
		
		   Note: textLook has no effect on squishy labels'''
		
		return self._text_look
		
	#def sends(self):
		#return self._sends
		

	# ---------------------------------


	def setLabel(self, label):
	  
		'''Set string for label diplayed '''
  
		self.setValue(label)
		return None


	# setValue method is overwritten to set label as well
	def setValue(self, val):
	  
		'''Set string for label diplayed '''
	  
		self._label = str(val)
		self._value = str(val)
		
		return None
		
	#def setLabelWidth(self, label_width):
	  
		#self._label_width = float(label_width)
		#return None
	      

	def setIsSquishy(self, true_or_false=True):
	  
		'''Set if the label is squishy or not
		
		       Squishy labels shrink smaller than full label size,
		       to show max chars (default 4) when space unavailable.
		   
		       Useful when user-defined/data-defined,
		       to prevent excessive wide labels.
		   
		   See also setSquichMax()'''
	  
		if bool(true_or_false)==True:
			self._label_type = uiScriptLabelType.squishy_label
			
		elif bool(true_or_false)==False:
			self._label_type =  uiScriptLabelType.label
			
		return None


	def setSquishMax(self, min_char_visible=None):
	  
		'''Return the max chars displayed for a squishy label
		
		       Squishy labels shrink smaller than full label size,
		       to show max chars (default 4) when space unavailable.
		   
		       Useful when user-defined/data-defined,
		       to prevent excessive wide labels.
		   
		   See also setIsSquishy()'''
	  
		self._squish_max = int(min_char_visible)
		return None


	def setTextLook(self, uiScriptTextLook):

		'''Returns a enumeration for the look of the text
		
		Note: textLook has no effect on squishy labels'''
  
		self._text_look = uiScriptTextLook
		return None

	#def setSends(self, int_val):
	  
		#if any([
		        #type(int_val) == type(float()),
		        #type(int_val) == type(int())
		       #]):
			 #self._sends = int(int_val)
		#return None

	# ---------------------------------

	def _printDeclarations(self):

		# value declaration 
		names = utils._gadgetTemplateNamesTuple(
		                                        self.name(),
		                                        self.namingScheme(),
		                                        self.numComponents()
		                                        )
		
		val_decl = names[0] + ':= "' + str(self._value) + '";'
		
		# gadget declaration
		gadget_decl = self._name + '.gad ='
		gadget_decl += ' '
		
		
		# its a squishy label or a normal label
		if self._label_type == uiScriptLabelType.squishy_label:
			gadget_decl += 'SQUISHY_'
		gadget_decl += 'LABEL'
		
		# squishy labels can have min number of chars
		if self._label_type == uiScriptLabelType.squishy_label:
			
			if self._printOkay(self._squish_max):
				gadget_decl += '(' + str(self._squish_max) + ')'
				
		# normal labels can have looks ie bold
		elif self._printOkay(self._text_look):
			
			gadget_decl += '(' + self._text_look.name() + ')'

		# add the actual label 
		#gadget_decl += ' "'+self._label +'"'
			
		#if self._printOkay(self._label_width):  
			#gadget_decl += ':' + str(self._label_width)

		gadget_decl += ' VALUE(' + self._name + '.val)'	

		all_attribs = self._printAllAttributes()
		cmds        = self._printCmds()

		result = val_decl +'\n'+ gadget_decl +' '+all_attribs+';\n'+cmds
		#result = gadget_decl +' '+all_attribs+';\n'+cmds

		return result

		

	##def _printSend(self):
		##return None
		###if self._printOkay(self._sends):
			###result = 'SENDS(' + str(self._sends) + ')'
		###else : result = None

		###return result 
		

	def _printGadgetAttributes(self):

		gadget_attribs = [
			          #self._printSend(),
			         ]
		result = [i for i in gadget_attribs if i !=None]
		return result
