# ==============================================================================
#               menu buttons used in uiScriptPy module
#  ----------------------------------------------------------------------------
#  Various Button Uiscript Feel via a button menuobject.
#      ACTION_MENU_BUTTON  ACTION_MENU_ICONBUTTON  
#      SELECT_MENU_BUTTON  SELECT_MENU_ICONBUTTON  
# ==============================================================================

from enums        import *
from base_classes import *

import utils

class MenuButtonUiScriptGadget(UiScriptGadget):

	def __init__(self, name,
	                   label       = None,
	                   menu_items  = ()
	            ):


		self.setName(name)
		self.setLabel(label)
		self.setMenuItems(menu_items)
		self._value = 0

		# label is opt, but as convention...
		if label == None: self._label = self._name

		return None


	#def __repr__(self):
		#pass

	# for declaration
	_type       = uiScriptGadgetType.menubutton
	_menu_type  = uiScriptMenuType.select

	# specific to this gadget
	_label        = None
	_label_width  = None
	_icon         = None

	_menu_uses_string_vals = False
	_menu_items = ()
	
	# access
	
	def menuType(self):
	  
		'''Return enumeration of menu type'''
		return self._menu_type
	      
	
	def icon(self):
	  
		'''Return the icon name if in use.''' 
		return self._icon

	def label(self):

		'''Return label diplayed as a string'''
		return self._label
	      

	def labelWidth(self):
	  
		'''Return the width of label displayed'''
		return self._label_width
		
	def menuUsesStringVals(self):
	  
		'''If menu button uses string value or not
		.
		Note. By default index values are return as int'''
		return self._menu_uses_string_vals

	def menuItems(self):
	  
		'''Return string tuple of menu items'''
		return self._menu_items

	# ---------------------------------
	
	# storage
	
	def setMenuType(self, menu_type = uiScriptMenuType.select):
	  
		'''Set to an enumeration of menu type'''
	  
		self._menu_type = menu_type
		return None


	def setIcon(self, iconname):

		'''Set the name icon file to use.''' 

		self._icon = iconname
		return None
	      

	def setLabel(self, label):
	  
		'''Set string for label diplayed '''

		self._label = label
		return None


	def setLabelWidth(self, label_width):

		'''Set the width of label displayed'''

		self._label_width = float(label_width)
		return None


	def setMenuUsesStringVals(self, bool_val):

		'''If menu button uses string value or not
		.
		Note. By default index values are return as int'''
		
		self._menu_uses_string_vals = bool(bool_val)
		return None
		

	def setMenuItems(self, menu_items):
	  
		'''Set string tuple of menu items'''
		
		self._menu_items = menu_items
		return None

	# ---------------------------------

	
	def _printDeclarations(self):

		# value declaration 
		names = utils._gadgetTemplateNamesTuple(
		                                        self.name(),
		                                        self.namingScheme(),
		                                        self.numComponents()
		                                        )
		
		val_decl = names[0] + ':= "' +  str(self._value) +'";'
		
		# menu declaration ...
		menu_decl = self._name + '.menu ='
		
		menu_type_attrib = ''
		
		# menu returns an index unless string have been flagged
		if self._menu_uses_string_vals == True:
			menu_type_attrib += ' STRING'
		else:
			menu_type_attrib += ' SELECT'
	
		# menubutton uses the icon version of the menu when icon is set 
		if self._printOkay(self._icon):
			menu_type_attrib += '_ICON'
	
		menu_type_attrib += '_MENU\n'
		
		# append the ATTRIB to the string
		menu_decl += menu_type_attrib
		
		# when using an icon, menu uses ICONTEXT pairs
		if self._printOkay(self._icon):
		  
			menu_items=[]
			for i in range(len(self._menu_items)):
			  
				if i%2 == 0: 
					menu_item = 'ICONTEXT( "' +self._menu_items[i]+ '", "' +self._menu_items[i+1]+ '" )\n'
					menu_items.append(menu_item)
					
		# without icon, menu is a string list
		else:
			menu_items = ['"' +x+ '"\n' for x in self._menu_items]
			
		# format the menu items and add sytax
		menu_items_string = '    ' + ('    '.join(menu_items) )
		menu_decl += '{\n' + menu_items_string + '}'

		
		# gadget declaration .....
		gadget_decl = self._name + '.gad ='

		# use the specified menu type in the declaration
		if self._menu_type == uiScriptMenuType.action:
			gadget_decl += ' ACTION_MENU'
		else: gadget_decl += ' SELECT_MENU'

		# when icon use is specified...
		if self._printOkay(self._icon):
		  
			# ... use it to munge the propper ATTRIB to decalre
			gadget_decl += '_ICONBUTTON'
			
			# select icon menus dont use a seperate icon
			if self._menu_type == uiScriptMenuType.action:
				gadget_decl +=  ' "'+ self._icon + '"'
			
		# for a normal non-icon button ...
		else:  
			# munge the normal ATTRIB to add and label
			gadget_decl += '_BUTTON'
			gadget_decl +=  ' "'+ self._label + '"'
			
			if self._printOkay(self._label_width):
				gadget_decl += ':' + str(self._label_width)
			
		
		# munge together the full declarations text with attribs, and cmds ...	
		gadget_decl += ' MENU(' + self._name + '.menu)'
		gadget_decl += ' VALUE(' + self._name + '.val)'	
		all_attribs = self._printAllAttributes()
		cmds        = self._printCmds()

		result = val_decl + '\n' +menu_decl + '\n' +gadget_decl +' '+all_attribs+';\n'+cmds

		return result


	#
	def _printSend(self):
		return None
		#if self._printOkay(self._layout):
			#result = result = 'LAYOUT(' + self._layout + ')'
		#else : result = None

		#return result
		

	def _printGadgetAttributes(self):

		gadget_attribs = [
			          self._printSend(),
			         ]
		result = [i for i in gadget_attribs if i !=None]
		return result
