# ==============================================================================
#                Number superclass of Int/Float used in uiScriptPy Lib
#  ----------------------------------------------------------------------------
#  Various label fields and related combines UI script ...
#     INT_FIELD INT_SLIDER INT_SLIDER_FIELD INT_VECTOR_FIELD INT_SPINNER_FIELD
#     FLOAT_FIELD FLOAT_SLIDER FLOAT_SLIDER_FIELD FLOAT_VECTOR_FIELD FLOAT_ SPINNER_FIELD
# ==============================================================================

import utils
from enums        import *
from base_classes import *


class NumUiScriptGadget(UiScriptGadget):

	def __init__(self, name, label=None, vector_size=None): 

		self.setName(name)
		self.setLabel(label)
		self.setVectorSize(vector_size)

		self._value       = 0
		self._type        = None

		# label is opt, but as convention...
		if not label: self._label = self._name

		return None

	#def __repr__(self):
		#pass

	# for declaration
	_type            = None
	_num_type        = uiScriptNumType.slider_field

	# specific to this gadget
	_label           = None
	_label_width     = None
	_increment_step  = None
	_vector_size     = None

	# specific gadget attibutes
	_range          = ()
	_lock_range_min = False
	_lock_range_max = False
	_rangevalue     = None
	#_sends        = None
	
	# ---------------------------------

	# access
	
	def label(self):
	  
		'''Return label diplayed as a string'''
		return self._label

		
	def labelWidth(self):
	  
		'''Return the width of label displayed'''
		return self._label_width

		
	def _hasSlider(self):
		
		if any([
		        self._num_type == uiScriptNumType.slider,
		        self._num_type == uiScriptNumType.slider_field
		      ]):
			result = True
		else:
			result = False
			
		return result

		
	def numType(self):
	  
		'''Returns enumration for number type'''
		return self._num_type
		
	def sliderRange(self):
	  
		'''Low and High usable range for the slider'''
		return self._range
	      
	def isSliderLockRangeMin(self):
	  
		'''Return if low of range is locked or not
		
		   Note User input will not exceed locked values'''
		   
		return self._lock_range_min
	      
	def isSliderLockRangeMax(self):
	  
		'''Return if high of range is locked or not
		
		   Note User input will not exceed locked values'''
		   
		return self._lock_range_max
	      
	def isSliderLockRange(self):
      
		'''Returns a tuple of bools representing,
		   if low and high of range is locked or not
		
		   Note User input will not exceed locked values'''
		   
		return tuple([self._lock_range_min, self._lock_range_max])

	      
	def sliderRangeValue(self):
		pass
	      
	def spinnerIncStepSize(self):
		return self._increment_step
		
	def vectorSize(self):
	  
		'''Return the number of vector componants as int'''
		return self._vector_size

	#def sends(self):
		#return self._sends
		

	# ---------------------------------
	

	#def setNumType(self, uiScriptNumType):
	  
		#self._num_type = uiScriptNumType
		#return None
		
		
	def setLabel(self, label):
	  
		'''Set string for label diplayed '''
		
		self._label = label
		return None
		
		
	def setLabelWidth(self, label_width):
	  
		'''Set the width of label displayed'''
	  
		if label_width == None:
		  
			self._label_width = None	
		else :
			self._label_width = float(label_width)
			
		return None
	      
	
	def setSliderRange(self, low, high):

		'''Set Low and High usable range for the slider'''

		self._range= (tuple([low,high]))
		return None

	      
	def setIsSliderLockRangeMin(self, true_or_false):
	  
		'''Set if low of range is locked or not
		
		   Note User input will not exceed locked values'''
		   
		self._lock_range_min = bool(true_or_false)
	      
	      
	def setIsSliderLockRangeMax(self, true_or_false):

		'''Set if high of range is locked or not
		
		   Note User input will not exceed locked values'''

		self._lock_range_max = bool(true_or_false)
	      
	     
	def setIsSliderLockRange(self, min_true_or_false, max_true_or_false):
	  
		'''Set if low and high of range is locked or not
		
		   Note User input will not exceed locked values'''
	  
		self._lock_range_min = bool(min_true_or_false)
		self._lock_range_max = bool(max_true_or_false)
		
		return None
	  
	# *** purpose undetermined
	#def setSliderRangeValue(self):
		#pass

	def setSpinnerIncStepSize(self, step_size):
	  
		self._increment_step = step_size
		return None

  
	def setVectorSize(self, vector_size):
	  
		'''Return the number of vector componants as int'''
		
		if any([
		        vector_size == None,
		        vector_size <= 1
		      ]):
			
			self.vector_size = None
			self._num_components = 1
			self._num_type = uiScriptNumType.slider_field
			
		else:
			self._num_type = uiScriptNumType.vector_field

			if vector_size > 4: 
				self._vector_size = 4
				self.__num_components = 4
			else:
				self._vector_size = int(vector_size)
				self._num_components = int(vector_size)
			
		return None
		
		
	#def setSends(self, int_val):
	  
		#if any([
		        #type(int_val) == type(float()),
		        #type(int_val) == type(int())
		       #]):
			 #self._sends = int(int_val)
		#return None

	# ---------------------------------

	def _printDeclarations(self):
	  
		# determine if a vector and size
		vector_size = None
		if self._num_type == uiScriptNumType.vector_field:
	  
			if all([
			       self._printOkay(self._vector_size),
			       self._vector_size >= 2
			      ]):
				vector_size = self._vector_size
			else: 
				vector_size = 3

				
		# value declaration ... get a list
		if self._printOkay(self._value):
			value = str(self._value).split(',')
			if len(value) != 1:
				value = str(self._value)[1:-1].split(',')
				
				
		# if its a bunk val, blank it or set to empty for vector
		elif vector_size:
			value = []
		else: value = ['']
		
		# value declaration
		names = utils._gadgetTemplateNamesTuple(
		                                        self.name(),
		                                        self.namingScheme(),
		                                        self.numComponents()
		                                        )
		if not vector_size:
			val_decl = names[0] + ':= ' +  str(value[0]) +';'
		
		else:  
			# a value declaration per vect element
			values = list(value)
			
			# ... ensure we meet the min legnth
			values.extend([0 for i in range(vector_size)])
			values = values[0:vector_size]

			# a value is declared per vector element
			val_decl = ''
			for i in range(len(values)):
				val_decl += names[i] + ':= ' + str(float(values[i])) + ';\n'

		# gadget declaration ...
		gadget_decl = self._name + '.gad ='
		
		# use the enum to dtermine the general gadget
		num_type = ' '
		num_type += self._type.name().upper()
		num_type += '_' + self._num_type.name().upper()
		gadget_decl += num_type

		# set the increment siz if its a spinner
		if self._num_type == uiScriptNumType.spinner_field:
			if self._printOkay(self._increment_step):
				gadget_decl += '(' + str(self._increment_step) + ')'
		
		# when a vector need the size
		if vector_size:
			gadget_decl += '(' + str(self._vector_size) + ')'

		# add the label if set
		if self._printOkay(self._label):
			gadget_decl += ' "'+self._label +'"'
		
			# print label width if required
			if self._printOkay(self._label_width): 
				gadget_decl += ':' + str(self._label_width)

		# value attrib need a ref per element for vector
		if not vector_size:
			gadget_decl += ' VALUE(' + names[0] +')'
		else:
			val_arg_str    = str( ', '.join(names) )
			gadget_decl   += ' VALUE(' + val_arg_str + ')'	
		
		# gather atributes
		all_attribs = self._printAllAttributes()
		cmds        = self._printCmds()

		result = val_decl +'\n'+ gadget_decl +' '+all_attribs+';\n'+cmds

		return result

		
		
	def _printSliderRange(self):
		
		result = None
	  
		if self._hasSlider() == True:

			if self._printOkay(self._range):
				result = 'RANGE' + str(self._range)
		return result


	def _printSliderRangeValue(self):
		
		result = None
	  
		if self._hasSlider() == True:

			if self._printOkay(self._rangevalue):
				result = 'RANGEVALUE' + str(self._rangevalue)
		return result

	
	def _printSliderLockRange(self):

		result = None
		if self._hasSlider() == True:

			if all([
			        self._lock_range_min != False,
			        self._lock_range_max != False,
				self._printOkay(self._lock_range_min),
				self._printOkay(self._lock_range_max)
			]):      
				result = 'LOCK_RANGE'

			elif self._lock_range_min != False:
				if self._printOkay(self._lock_range_min):
					result = 'LOCK_RANGE_MIN'

			elif self._lock_range_max != False:
				if self._printOkay(self._lock_range_max):
					result = 'LOCK_RANGE_MAX'
				
		return result

		
	##def _printSend(self):
		##return None
		###if self._printOkay(self._sends):
			###result = 'SENDS(' + str(self._sends) + ')'
		###else : result = None

		###return result 
		

	def _printGadgetAttributes(self):

		gadget_attribs = [
			          #self._printSend(),
			          self._printSliderRange(),
			          self._printSliderLockRange(),
			          self._printSliderRangeValue()
			         ]
		result = [i for i in gadget_attribs if i !=None]
		return result
