# ==============================================================================
#           scroller containers class used in uiScriptPy module
#  ----------------------------------------------------------------------------
#  scroller uiScriptContainer currently either for ui Script attributes
#       SCROLLER and TOOLSSCROLLER
# ==============================================================================

from   enums        import *
import utils
from   base_classes import *
from   container_class import*

class ScrollerUiScriptContainer(UiScriptContainer):



	def __init__(self, name,
	                   uiScriptGadgets=(),
	                   direction=uiScriptDirection.vertical
	                   ):

		self.setName(name)
		self.setUiScriptGadgets(uiScriptGadgets)
		
		self._container_type = uiScriptContainerType.scroller
		
		return None
		
	# for declaration
	_children       = ()
	_scroller_type  = uiScriptScrollerType.scroller
	_container_type = uiScriptContainerType.scroller
	_type           = uiScriptGadgetType.container


	# specific to this gadget
	_direction        = None
	_is_tool_scroller = False
	
	# specific gadget attibutes
	# NONE

	# ---------------------------------
	# access
	

	def direction(self):
	  
		'''Retrun enumeration for scroll direction(s)
		
		Note. Tool Scrollers scroll in one direction only'''
		
		return self._direction


	def isToolScroller(self):
	  
		'''Return if ta tool scroller or not.
		
		Tool Scrollers do not have a scrollbar.
		Instead, arrows appear indicating additonal content.
		
		Note. Tool Scrollers scroll in one direction only'''
		
		return self._is_tool_scroller
	
	## ---------------------------------
	

	def setDirection(self, uiScriptDirection):

		'''Retrun enumeration for scroll direction(s)
		
		Note. Tool Scrollers scroll in one direction only'''
		

		self._direction = uiScriptDirection
		self._syncDirectionAndLayout()
		
		return None


	def setIsToolScroller(self, true_or_false):
	  
		'''Set if ta tool scroller or not.
		
		Tool Scrollers do not have a scrollbar.
		Instead, arrows appear indicating additonal content.
		
		Note. Tool Scrollers scroll in one direction only'''
		
		self._is_tool_scroller = bool(true_or_false)
		
		if self._is_tool_scroller == True:
			self._scroller_type = uiScriptScrollerType.toolscroller
		else:
			self._scroller_type = uiScriptScrollerType.scroller
		self._syncDirectionAndLayout()

		return None

	# ---------------------------------
	
	def _syncDirectionAndLayout(self):
	
		# layout (inherited) + direction have some ugly results
		#    ie vert scroll, and horiz layout don't mesh
		#    while normal scroller uses direction,
		#    tool scollers use layout.
		# This mess needs to be cleaned first.
		
		if self._scroller_type == uiScriptScrollerType.toolscroller:
		  
			# move direction to layout (sans "full") 
			if self._direction == uiScriptDirection.vertical:
				self._layout = uiScriptLayout.vertical
			elif self._direction == uiScriptDirection.horizontal:
				self._layout = uiScriptLayout.horizontal
			else:
				self._layout = None
				
		elif self._scroller_type == uiScriptScrollerType.scroller:
		  
			# when a normal scroller, blank the layout
			# the rare case of horz scroller and layout
			# can be handled by scroller/row layouts
			self._layout = None
			
		return None
	
	def _printDeclarations(self):
	
		# value declaration
		# NONE scollers do not send or receive values

		gadget_decl = self._name + '.gad ='
		
		scroller_type = self._scroller_type.name().upper()
		gadget_decl +=  ' ' + scroller_type
		
		if self._scroller_type == uiScriptScrollerType.scroller:
		  
			# set the direction vian enum for non-toolbars
			if self._printOkay(self._direction):
			  
				direction = self._direction.name()
				gadget_decl += ' ' +direction
		
		all_attribs = self._printAllAttributes()
		all_attribs = '    ' + all_attribs
		
		gadget_refs = [x._name + '.gad;\n' for x in self._children]
		gadget_refs_string = '    ' + ('    '.join(gadget_refs) )
		
		gadget_decl += '\n{\n' + all_attribs + '\n\n' + gadget_refs_string + '}'

		# commands
		cmds        = self._printCmds()

		# resulting string
		result = gadget_decl +'\n'+cmds
		
		return result