# ==============================================================================
#                String used in uiScriptPy Lib
#  ----------------------------------------------------------------------------
#  Various String fields and related combines US script ...
#      STRING_FIELD ALPHANUM_FIELD PASSWORD_FIELD STRING_LIMITED_FIELD
#      STRING_MENU_FIELD
# ==============================================================================

from enums        import *
from base_classes import *

import utils


class StringUiScriptGadget(UiScriptGadget):

	def __init__(self, name,
	                   label       = None,
	                   menu_items  = (),
	                   string_type = uiScriptStringType.string
	            ):

		self.setName(name)
		self.setLabel(label)
		self.setMenuItems(menu_items)
		self._value   = ''

		# label is opt, but as convention...
		if label == None: self._label = self._name

		return None


	#def __repr__(self):
		#pass

	# for declaration
	_type        = uiScriptGadgetType.string
	_string_type = uiScriptStringType.string

	# specific to this gadget
	_label               = None
	_label_width         = None
	_limit_to_chars      = None
	_menu_items          = None
	_string_replace_type = None

	# specific gadget attibutes
	# none yet....
	
	# access
	def stringType(self):
	  
		'''Returns enumeration for the string type'''
		return self._string_type
	
	def label(self):
	  
		'''Return label diplayed as a string'''
		return self._label

	def labelWidth(self):
	  
		'''Return the width of label displayed'''
		return self._label_width
		
	def isAlphaNum(self):
	  
		'''Return if the string field is alpha-numeric or not.
		
		    Alpha-numeric fileds only accept letters and numbers.
		    All other entered chars transform to underscores (_).
		
		Note. This applies to user-entered data ONLY'''
	  
		if self._string_type == uiScriptStringType.alphanum:
			result = True
		else: result = False
		
		return result
	      
	def isPassword(self):
	  
		'''Return if field displays chars as password.
		
		    All other entered chars transform to underscores (_).
		
		Note. Passwords are still retrievable as plain text'''
	  
		if self._string_type == uiScriptStringType.password:
			result = True
		else: result = False
		
		return result

	def isLimitedToChars(self):

		'''Return if field is limited to specific chars'''
		
		result = None
		if self._limit_to_chars == None:
			result = False
		else:
			result = True
			
		return result
		

	def limitToChars(self):
	  
		'''Return a list of chars this field is limited.
		
		Note. This applies to user-entered data ONLY'''

		return self._limit_to_chars
	      
	def menuItems(self):
	  
		'''Return string tuple of menu items'''
		return tuple(self._menu_items)


	def hasMenu(self):
	  
		'''Return field string field-menu combo or not.
		    Menu will populate or append the field.
		    
		This is a convieniance and same as setting the enum
		
		Note. Passwords are still retrievable as plain text
		Note. password, alphaNum, and string_menu
		    are enums and mutually exclusive.
		    
		see also...  stringReplaceType()'''
		
		result = None
		if self._string_type == uiScriptStringType.string_menu:
			result = True
		else:
			result = False
			
		return result
    
	def stringReplaceType(self):
	  
		'''Enumeration of the replacent method.
		
		Note only valid for string-menus.'''
		
		return self._string_replace_type

	# ---------------------------------
	
	def setStringType(self, uiScriptStringType):
	  
		'''set a string type enumeration.
		    Set field stylefor passwords, alpa numeric,
		    or to use a menu.
		    
		Note. limiting chars takes PRIORITY over this.
		
		see also...  setIstAplhaNum()
		             setIsPassword()
		             setLimitToChars()
		             setHasMenu()'''
		
		self._string_type = uiScriptStringType
		return None


	def setLabel(self, label):
	  
		'''Set string for label diplayed '''

		self._label = label
		return None

		
	def setLabelWidth(self, label_width):

		'''Set the width of label displayed'''

		self._label_width = float(label_width)
		return None

	def setIsAplhaNum(self, true_or_false=True):
	  
		'''Set field to only accept letters and numbers.
		All other entered chars transform to underscores (_).
		
		This is a convieniance and same as setting the enum
		
		Note.  This applies to user-entered data ONLY.
		Note. password, alphaNum, and string_menu
		    are enums and mutually exclusive.'''
		    
		if true_or_false == True:
			self._string_type = uiScriptStringType.alphanum
		else:
			self._string_type = uiScriptStringType.string
			
		return None
	  
	def setIsPassword(self, true_or_false=True):
	  
		'''Set Field to be used for passwords.
		Password will obscure the string entered.
		
		This is a convieniance and same as setting the enum
		
		Note. Passwords are still retrievable as plain text
		Note. password, alphaNum, and string_menu
		    are enums and mutually exclusive.'''
		    
		if true_or_false == True:
			self._string_type = uiScriptStringType.password
		else:
			self._string_type = uiScriptStringType.string
			
		return None
		
	
	def setLimitToChars(self, limit_to_chars):
	  
		'''Set list of valid characters that can be entered.
		All other entered chars transform to underscores (_).

		    Ranges may be used ("A-z", "0-9").
		    
		Note. This takes PRIORITY over string type enum
		
		Note. This applies to user-entered data ONLY
		Note. To use '-', backslash it as... "\-" '''
		
		self._limit_to_chars = tuple(limit_to_chars)
		return None


	def setMenuItems(self, menu_items):
	  
		'''Set string tuple for menu items'''
 
		self._menu_items = menu_items
		return None


	def setHasMenu(self, true_or_false=True):
	  
		'''Set if is a string field-menu combo or not.
		    Menu will populate or append the field.
		    
		This is a convieniance and same as setting the enum
		
		Note. Passwords are still retrievable as plain text
		Note. password, alphaNum, and string_menu
		    are enums and mutually exclusive.
		    
		see also...  setStringReplaceType()'''
  
		if true_or_false == True:
			self._string_type = uiScriptStringType.string_menu
		else:
			self._string_type = uiScriptStringType.string
			
		return None



	def setStringReplaceType(self, uiScriptStringReplaceType):
	  
		'''Enumeration of the replacent method.
		
		Note only valid for string-menus.'''

		self._string_replace_type = uiScriptStringReplaceType
		return None

	# ---------------------------------

	def _printDeclarations(self):

		# value declaration 
		names = utils._gadgetTemplateNamesTuple(
		                                        self.name(),
		                                        self.namingScheme(),
		                                        self.numComponents()
		                                        )
		
		val_decl = names[0] + ':= "' +  str(self._value) +'";'
		
		# menu declaration
		menu_decl = ''
		if self._string_type == uiScriptStringType.string_menu:
		
			# menu declaration
			menu_decl = self._name + '.menu ='
			
			menu_decl += ' STRING_MENU\n'
				
			menu_items = ['"' +x+ '"\n' for x in self._menu_items]
			menu_items_string = '    ' + ('    '.join(menu_items) )
			
			menu_decl += '{\n' + menu_items_string + '}'


		# gadget declaration
		gadget_decl = self._name + '.gad ='
		gadget_decl += ' '
		
		# this is a limited string field 
		if self._printOkay(self._limit_to_chars):
		  
			limited_to = self._limit_to_chars
			limited_to = ', '.join( [str(x) for x in list(limited_to)] )
			gadget_decl += 'STRING_LIMITED_FIELD("' + limited_to + '" )'
		
		# its a password field ---technically not a string
		elif self._string_type == uiScriptStringType.password:
			gadget_decl += 'PASSWORD_FIELD'
			
		# use the enum to set as a string, alphanum
		else :
			string_type = self._string_type.name().upper() + '_FIELD'
			gadget_decl += string_type
			
		# check for replacement type when using a men
		if all([
		        self._string_type == uiScriptStringType.string_menu,
		        self._printOkay(self._string_replace_type)
		      ]):
			gadget_decl += '(' + self._string_replace_type.name() + ')'

		# add the optional label if set
		if self._printOkay(self._label):
			gadget_decl += ' "'+self._label +'"'
	
			# print label width if required
			if self._printOkay(self._label_width):  
				gadget_decl += ':' + str(self._label_width)
		
		# add a menu ref if a string_menu
		if self._string_type == uiScriptStringType.string_menu:
			gadget_decl += ' MENU(' + self._name + '.menu)'
		
		gadget_decl += ' VALUE(' + self._name + '.val)'

		all_attribs = self._printAllAttributes()
		cmds        = self._printCmds()

		result =  menu_decl + '\n' + val_decl +'\n'+ gadget_decl +' '+all_attribs+';\n'+cmds

		return result


	def _printGadgetAttributes(self):

		gadget_attribs = [
			          #self._printSmoo(),
			         ]
		result = [i for i in gadget_attribs if i !=None]
		return result
