import utils
import hou

class Gadget(object):
  
	#def __init__ (self, **kwargs):
	def __init__ (self, hou_dialog, ui_script_gadget, comp_index=0):

		# expects a ui_script_gadget  hou_dialog
		#hou_dialog = kwargs['hou_dialog']
		#ui_script_gadget = kwargs['ui_script_gadget']
		

		# size and naming styple of tuple
		self._numcomponants = ui_script_gadget.numComponents()
		self._naming_scheme = ui_script_gadget.namingScheme()
	
		# ensure we get to a tuple of propper legnth.
		# this is to compensaet for changing comp legnth during
		# ui script design, and done on num_class during print
		ui_script_val = ui_script_gadget.value()
		
		if isinstance(ui_script_val, tuple):
			ui_script_val = list(ui_script_val)
		else:
			ui_script_val = list([ui_script_val])

		# ... ensure we meet the min legnth
		ui_script_val.extend([0 for i in range(self._numcomponants)])
		ui_script_val = tuple(ui_script_val[0:self._numcomponants])

		# name depends on comonant size
		comp_names = utils._gadgetNamesTuple(
		                                     ui_script_gadget.name(),
		                                     self._naming_scheme,
		                                     self._numcomponants
		                                    )
		# error out if any ... 
		if comp_index >= len(comp_names):
			# let someone know the goofed up  
			raise hou.ValueError('componant index out of range')

		# all is good set the name 
		self._name = comp_names[comp_index]

		# figure out and store corresponding ui script tempalte name
		ui_script_names = utils._gadgetTemplateNamesTuple(
		                                                  ui_script_gadget.name(),
		                                                  self._naming_scheme,
		                                                  self._numcomponants
		                                                 )
		self._ui_script_name = ui_script_names[comp_index]
		
		# default has already been padded and ensure tuple
		self._default = ui_script_val[comp_index]
		
		# description can be from a few places
		if  hasattr(ui_script_gadget, '_title'):
			self._description = ui_script_gadget._title
		elif hasattr(ui_script_gadget, '_label'):
			self._description = ui_script_gadget._label
		else:  gadget._description = None
		
		# get pointers to parents
		self._dialog = hou_dialog
		self._uiscript = ui_script_gadget
		
		# add callbacks
		self._ui_script_callback = ui_script_gadget.scriptCallback()
		# check and account for storing template callbacks
		if self._ui_script_callback != None:
			self._has_ui_script_callback = 1


	_uiscript               = None
	_default                = None
	_description            = None
	_name                   = None
	_ui_script_name         = None
	_dialog                 = None
	_numcomponants          = None
	_naming_scheme          = None
	_callback               = None
	_ui_script_callback     = None
	_has_ui_script_callback = False
	
	def name(self):
	  
		'''Returns gadget name'''
		return self._name
	      
	def set(self, value):
	  
		'''Set Gadget to specified Value'''
		
		hou_dialog = self.dialog()
		result = hou_dialog.setValue(self._ui_script_name, value)

		return result
	      
	def disable(self, true_or_false=True):
	  
		'''Show the Gadget as disabled (ghosted) in dialog'''
	  
		true_or_false = bool(true_or_false)
		dialog = self.dialog()
		# invert as houdini dialog only use enable
		dialog.enableValue(self._ui_script_name, not true_or_false)
		
		return None
	      
	def enable(self, true_or_false=True):
	  
		'''Show the Gadget as enabled (not ghosted) in dialog'''
		
		true_or_false = bool(true_or_false)
		dialog = self.dialog()
		dialog.enableValue(self._ui_script_name, true_or_false)

		return None
	      
	def get(self):
	  
		'''Retrun the current Gadget value'''
		
		hou_dialog = self.dialog()
		result = hou_dialog.value(self._ui_script_name)
		
		return result
		
		
	def value(self):

		'''alias for get'''
		return self.get()
	      
	def isAtDefault(self):
	  
		# get then compare to template value
		
		result =None
		if self._default == self.get():
			result = True
		else: 
			result = False
		return result
      
	def asCode(self):
		pass
	      
	def description(self):
	  
		'''Returns Gadget label ot title'''
		return self._description
	      
	def uiScriptGadget(self):
	  
		'''Return template uiScriptGadget'''
		return self._uiscript
	      
	def dialog(self):
	
		'''Returns the dialog containing this Gadget'''
		return self._dialog
   
	def hasInstanceCallback(self):
	  
		'''Retruns if an Instange Callback is atatched or not'''
		return self._has_ui_script_callback
	      
	def clearInstanceCallback(self):
	  
		'''Clears callback script this Gadget'''
	
		# only clear call back added to the instance
		# in future multi callbacks may be allowedo n the template
		# in all cases can count backward
		
		dialog = self.dialog()
		ui_script_names = []
		
		# check and deal with multi componant gadgetsTuples
		if isinstance(self._ui_script_name, tuple):
			
			# remove callback per componant
			ui_script_names = list(self._ui_script_name)

		# its a single componant tuple or non tuple
		else: ui_script_names = [self._ui_script_name]

		# get rid of the call back now that we know where
		for ui_script_name in ui_script_names:

			# min legneth depends on ui script callback presence
			# longer than require by template means instance callback
			# will NOT work with multi/multilang -- REVISIT
			all_callbacks = dialog.callbacks(ui_script_name)
			if len(all_callbacks) > self._has_ui_script_callback:

				# count backwarsd to leave template undisturbed
				dead = dialog.callbacks(ui_script_name)[-1]
				dialog.removeCallback(ui_script_name, dead)

			# clear the gadget's storage
			self._callback = None

		return None


	def setInstanceCallback(self, callback_string):
	  
		''' Set the callback script to provided string.
		This script will run in response to gadget changes.
		
		Note Currently only python callbacks are supported.
		     hscript can be used via hou.hscript()'''
		     
		callback_string = str(callback_string)
		ui_script_names = []

		# check and deal with multi componant gadgetsTuples
		if isinstance(self._ui_script_name, tuple):
			
			# callback per componant
			ui_script_names = list(self._ui_script_name)
				
		# its a single componant tuple or non tuple
		else: ui_script_names = [self._ui_script_name]
		
		# clear pre-existing callback
		self.clearInstanceCallback()
		
		# add callback per ui script name associated with gadget/tuple
		for ui_script_name in ui_script_names:
			print 'setting... ' + ui_script_name
			self._processCallback(
			                      callback=callback_string,
			                      ui_script_name=ui_script_name,
			                      store=True)
		
		self._callback = callback_string
		
		return None


	def _processCallback(self, **kwargs):
  
		# kwargs
		# callback_script = string
		# ui_script_name = string name ui script gadget/compnant name
		# store = bool (callback fro mthe template are not)
		# `       ** when not present False

		callback = str(kwargs['callback'])
		ui_script_name = kwargs['ui_script_name']
		if kwargs.has_key('store'):
			store = kwargs['store']
		else: store = False
		
		filename = ui_script_name.replace('.', '_')
		dialog = self.dialog()
  
		dialog.addCallback(
		                   ui_script_name,
		                   lambda: eval(compile(callback,filename,'exec'))
		                  )

		# store if on the gadget instance
		if store == True:
			self._callback = callback

		return None
 


class GadgetTuple(Gadget):
  
	#def __init__ (self, **kwargs):
	def __init__ (self, hou_dialog, ui_script_gadget):

		# expects a ui_script_gadget  hou_dialog
		# hou_dialog = kwargs['hou_dialog']
		# ui_script_gadget = kwargs['ui_script_gadget']
		
		
		# size and naming styple of tuple
		self._numcomponants = ui_script_gadget.numComponents()
		self._naming_scheme = ui_script_gadget.namingScheme()
		
		# ensure we get to a tuple of propper legnth.
		# this is to compensate for changing comp legnth during
		# ui script design, and done on num_class during print
		ui_script_val = ui_script_gadget.value()
		
		if isinstance(ui_script_val, tuple):
			ui_script_val = list(ui_script_val)
		else:
			ui_script_val = list([ui_script_val])

		# ... ensure we meet the min legnth
		ui_script_val.extend([0 for i in range(self._numcomponants)])
		ui_script_val = tuple(ui_script_val[0:self._numcomponants])
		
		# name 
		self._name = ui_script_gadget.name()
		
		# figure out and store corresponding ui script tempalte name
		ui_script_names = utils._gadgetTemplateNamesTuple(
		                                                  ui_script_gadget.name(),
		                                                  self._naming_scheme,
		                                                  self._numcomponants
		                                                 )
		self._ui_script_name = ui_script_names
		
		# default has already been padded and ensure tuple
		self._default = ui_script_val

		# description can be from a few places
		if  hasattr(ui_script_gadget, '_title'):
			self._description = ui_script_gadget._title
		elif hasattr(ui_script_gadget, '_label'):
			self._description = ui_script_gadget._label
		else:  gadget._description = None
		
		# get pointers to parents
		self._dialog = hou_dialog
		self._uiscript = ui_script_gadget
		
		# store ui script template callback
		self._ui_script_callback = ui_script_gadget.scriptCallback()
		if self._ui_script_callback != None:
			self._has_ui_script_callback = 1


	def set(self, value):
	
		'''Set Gadget to specified Value Tuple'''
		
		# safety check..
		if isinstance(value, tuple):
			if self._numcomponants != len(value):	             
				# need to trow an error
				pass
		
		hou_dialog = self.dialog()
		ui_script_names = self._ui_script_name

		# set componant values
		result = None
		for i in range(self._numcomponants):
		  
			ui_script_name = ui_script_names[i]
			result = hou_dialog.setValue(ui_script_name, value[i])

		return result


	def disable(self, true_or_false=True):
	  
		'''Show the Gadget as disabled (ghosted) in dialog'''
	  
		true_or_false = bool(true_or_false)
		dialog = self.dialog()
		
		# invert as houdini dialog only use enable
		# looping over each componant
		for ui_script_name in list(self._ui_script_name):
			dialog.enableValue(ui_script_name, not true_or_false)
		
		return None
	      
	def enable(self, true_or_false=True):
	  
		'''Show the Gadget as enabled (not ghosted) in dialog'''
		
		true_or_false = bool(true_or_false)
		dialog = self.dialog()

		# looping over each componant
		for ui_script_name in list(self._ui_script_name):
			dialog.enableValue(ui_script_name, true_or_false)
		
		return None

	def get(self):

		'''Retrun the current Gadget value'''

		hou_dialog = self.dialog()
		ui_script_names = self._ui_script_name
		
		# pull tuple gadget template values
		ui_script_gadget = self.uiScriptGadget()

		# build a tuple from value query
		result = ()
		for ui_script_name in ui_script_names:
	
			result += tuple([hou_dialog.value(ui_script_name)])

		return result

