# ==============================================================================
#           dialog/window container class used in uiScriptPy module
#  ----------------------------------------------------------------------------
#  Establishes the DialogUiScriptWindow and classe
#       this is analogous to parmTemplateGroup for parm tmeplates
#       functionality added for managing gadgets in containers
#       and for dialog generation.
# ==============================================================================

import tempfile
import copy

from enums           import *
from hdialog_extend  import _extendHouDialog
import               utils

from container_class import *


class DialogUiScriptWindow(UiScriptContainer):

	def __init__(self, name, title=None, uiScriptGadgets=(), dialog=True):

		
		self.setName(name)
		self.setTitle(title)
		self.setUiScriptGadgets(uiScriptGadgets)
		
		# dialog or normal window
		self.setIsDialog(dialog)
		
		# accomadates min close button
		self.setMinWidth(1)
		
		# keeps windows edges from too near
		self.setHMargin(0.1)
		self.setVMargin(0.1)
		self._val = name + '.val'

		return None
	      
	_type          = uiScriptGadgetType.window
	_window_type   = None
	#_val           = _name + '.val'
	#_val            = 0
	_look          = uiScriptLook.plain
	
	# specific to this gadget
	_icon          = None
	_title         = None 
	_default_title = 'uiScriptPy'

	# specific gadget attibutes
	_exclusive     = False

	# ---------------------------------

	def isDialog(self):
	  
		result = False
		if self._window_type == uiScriptWindowType.dialog:
			result = True
		  
		return result

			
	def title(self):
		return self._title

		
	def icon(self):
		return self._icon

		
	def exclusive(self):
		return self._exclusive

	# ---------------------------------
	
	def setIsDialog(self, value):
		
		if bool(value) == True:
			 self._window_type = uiScriptWindowType.dialog
		else:
			 self._window_type = uiScriptWindowType.normal
			
		return None

		
	def setTitle(self, title):
		
		if title == None:
			self._title = self._default_title
		else:
			self._title = str(title)
		return None

		
	def setIcon(self, iconname):
		
		self._icon = iconname
		return None

		
	def setExclusive(self, value):
		
		self._exclusive = bool(value)
		return None

	# ---------------------------------
	
	#def containers(self):
	
		#'''Returns all UI Script Container gadgets'''
	
		#all_entries = self.entries()
		#all_containers = tuple ( [x for x in all_entries if x])
		
		#return all_containers	


	def _flatGadgetsByType(self, search_objects = [ {'gadget' : None, 'indices' : (0,)} ]):
	
	# Expects a dictionary each being a list per key windows, containers, gadget
	# List object is another dict mapping the gadget and indices (path to gadget)
	# derfault is to start ans esearch only this window downward

		gadgets = []
		containers = []
		windows = []
		
		# when searching only parent is len 1, single dict
		# allowing room to expand later to searching multiple
		# gadget collection dialogs etc ...
		parents = list(copy.copy(search_objects))
		
		# BAD test and why it fgoes down 1 level
		#while len(containers) > 0:
		monkey = True
		while monkey == True and len(parents) > 0:
			monkey =False
			
			for parent in parents:
			#for parent in search_objects:
			  
				# grab the window when inital recrusion
				if parent['gadget'] == None:
					parent['gadget'] = self
					parent['indices'] = ()
					windows.append(parent)
				
				# for each gadget in this dialog/window/container
				children = parent['gadget'].uiScriptGadgets()
				
				for i in range(len(children)):
					
					# a dict per child, child indices
					# indices is an index based gadget path
					#  ...and a confusing houdini convention
					child = children[i]
					child_dict = {
					              'gadget'  : child,
					              'indices' : parent['indices'] + (i,)
					              }

					if child.type()==uiScriptGadgetType.container:
						containers.append(child_dict)
						
					elif child.type()!=uiScriptGadgetType.window:
						gadgets.append(child_dict)
						
					else: continue
				                                  
				next_level_search_objects = tuple(containers)
				next_level_results = self._flatGadgetsByType(next_level_search_objects)
				containers.extend(next_level_results['containers'])
				gadgets.extend(next_level_results['gadgets'])

				parents.remove(parent)
				
		sorted_gadgets = {
			          'windows'      :  tuple(windows),
			          'containers'   :  tuple(containers),
			          'gadgets'      :  tuple(gadgets)
			         }
				
		return sorted_gadgets
	
	# ---------------------------------

	# manage gadgets


	def append(self, UiScriptGadget):
		
		'''Add a UI Script Gadget after all existing UI Script Gadgets'''

		self.addUiScriptGadget(UiScriptGadget)
		
		return None


	def appendToContainer(self, name_or_container_or_indices, gadget_or_container):
	
		'''Add a UI Script Gadget on the end of Container contents.'''

		# its the name
		if type(name_or_container_or_indices) == type(''):
			container = self.findContainer(name_or_container_or_indices)
			
		# its the index path
		elif any([
		        type(name_or_container_or_indices)==type(()),
		        type(name_or_container_or_indices)==type([])
		       ]):
			container = self.entryAtIndices(tuple(name_or_container_or_indices))

		# we have a gadget already
		else: container = name_or_container_or_indices
			
		# its a container the append
		if container.type() == uiScriptGadgetType.container:
			container.addUiScriptGadget(gadget_or_container)

		return None


	def asCode(self): ## doable now?
	
		'''Return Python code that will recreate this container.'''
		pass
	
	def asDialogScript(self):  ## not sure if this is possible either
		
		'''Return a string containing the dialog script file contents corresponding to this UI Script Gadget Container'''
		pass


	def clear(self):
		
		'''Remove all  UI Script Gadgets from this container'''
		
		self._children = ()
		return None


	def containingContainer(self, name_or_gadget):
		
		'''Return UiScriptContainer in this window containing a UI Script Gadget.'''
		
		parent_container_indices = self.containingContainerIndices(name_or_gadget)
		parent_container = self.entryAtIndices(parent_container_indices)
		
		return parent_container


	def containingContainerIndices(self, name_or_gadget_or_indices):
		
		'''Return the indices of the container in this window containing a given UI Script Gadget.
		   Return an empty tuple if no match was found. '''
		   
		# its a anme or gadget
		if not any([
		        type(name_or_gadget_or_indices)==type(()),
		        type(name_or_gadget_or_indices)==type([])
		       ]):

			gadget_indices = self.findIndices(name_or_gadget_or_indices)

		# we have typle already
		else:
			gadget_indices = name_or_gadget_or_indices
			
		# remove the child index to get parent
		parent_container_indices = gadget_indices[:-1]

		return parent_container_indices


	def entries(self):
		
		'''Return a tuple containing all UiScriptGadgets inside this window.'''
		
		return self.uiScriptGadgets()
		

	def entriesWithoutContainers(self):
		
		'''Return a tuple containing all the non-container UI Script Gadgets inside this group.
		   The output is the flattened set of UI Script Gadgets in the window, 
		       and includes the non-container gadgets inside containers. '''
		       
		sorted_gadgets = self._flatGadgetsByType()
		gadgets = [ x['gadget'] for x in sorted_gadgets['gadgets'] ]
		
		return gadgets
	
	
	def entryAtIndices(self, indices):
		
		'''Given a sequence of integer indices return a copy of the UiSctiptGadget object at that location.
		   The indices are of the format returned by findIndices'''
		
		cmd = 'self' 
		
		for idx in indices:
			cmd += '.uiScriptGadgets()[' + str(idx) + ']'
			UiScriptContainer
		result = eval(cmd)
		return result


		
	def find(self, name):
		
		'''Search in this container for a UI Script Gadget whose name matches the specified one.
		   Return a copy of the  UI Script gadget or None if no match was found.'''
		   
		sorted_gadgets = self._flatGadgetsByType()
		all_gadgets = sorted_gadgets['containers']+sorted_gadgets['gadgets']
		
		result = None
		for gadget in all_gadgets:
			
			if gadget['gadget'].name() == name:
				result = gadget['gadget']
				break
			else: continue
			
		return result


	def findContainer(self, name):
	
		'''Search in this group for a container UI Script gadget with the given name.
		   Return the UiScriptContainer or None if the container wasn't found.'''
		
		sorted_gadgets = self._flatGadgetsByType()
		containers = sorted_gadgets['containers']
		
		result = None
		for container in containers:
		  
			if container['gadget'].name() == name:
				result = container['gadget']
			else:
				continue

		return result

	def indicesOf(self, name_or_gadget):
	  
		 '''earch in this window for a UI Script gadget and return a tuple of integers corresponding
		       to the indices of the gadget in the window. Return an empty tuple if no match was found.

		 Alias for findIndices'''
		 return self.findIndices(name_or_gadget)

		
	def findIndices(self, name_or_gadget):
		
		'''Search in this window for a UI Script gadget and return a tuple of integers corresponding
		       to the indices of the gadget in the window. Return an empty tuple if no match was found.'''
			
		sorted_gadgets = self._flatGadgetsByType()
		all_gadgets = sorted_gadgets['containers'] + sorted_gadgets['gadgets']
		
		result = ()
		for gadget in all_gadgets:
			
			if any([
				gadget['gadget'].name() == name_or_gadget,
				gadget['gadget']        == name_or_gadget
				]):
					
				result = gadget['indices']
				break
				
			else: continue
			
		return result
		
	
	def findIndicesForContainer(self, name):
	#def findIndicesForContainer(self, label_or_labels):
		
		'''Search in this window for a container UI Script gadget with the given label.
		   Return an empty tuple if no match was found.'''

		sorted_gadgets = self._flatGadgetsByType()
		containers = sorted_gadgets['containers']
		
		result = ()
		for container in containers:
			
			if container['gadget'].name() == name:

				result = container['indices']
				break
				
			else: continue
			
		return result
		
	
	def hide(self): ## can it be done
		
		'''Hide a gadget by making it invisible.
		   The gadget still exists in the window, but will not  be display in it.'''
		pass
	
	def hideContainer(self): ## not sure about this
		
		'''Hide a container and all the gadget in it'''
		pass


	def insertAfter(self, name_or_gadget_or_indices, gadget_or_container):
		
		'''Insert a UI Script Gadget inside this window after an existing one.
		   The new gadget will be in the same container as the existing one. '''

		# its a anme or gadget
		if not any([
		        type(name_or_gadget_or_indices)==type(()),
		        type(name_or_gadget_or_indices)==type([])
		       ]):

			gadget_indices = self.findIndices(name_or_gadget_or_indices)

		# we have typle already
		else:
			gadget_indices = name_or_gadget_or_indices
			
		# perform the edit
		parent      = self.entryAtIndices(gadget_indices[:-1])
		sibling     = self.entryAtIndices(gadget_indices)
		children    = list(parent.uiScriptGadgets())
		sibling_idx = children.index(sibling)

		# ( by default insert(), insets BEFORE the index
		children.insert(sibling_idx + 1, gadget_or_container)
		children = tuple(children)
		
		# set it back to parent container
		parent.setUiScriptGadgets(children)
		
		return None


	def insertBefore(self, name_or_gadget_or_indices, gadget_or_container):
		
		'''Insert a UI Script Gadget inside this window before an existing one.
		   The new gadget will be in the same container as the existing one.'''
	
		# its a anme or gadget
		if not any([
		        type(name_or_gadget_or_indices)==type(()),
		        type(name_or_gadget_or_indices)==type([])
		       ]):

			gadget_indices = self.findIndices(name_or_gadget_or_indices)

		# we have typle already
		else:
			gadget_indices = name_or_gadget_or_indices
			
		# perform the edit
		parent      = self.entryAtIndices(gadget_indices[:-1])
		sibling     = self.entryAtIndices(gadget_indices)
		children    = list(parent.uiScriptGadgets())
		sibling_idx = children.index(sibling)

		# ( by default insert(), insets BEFORE the index
		children.insert(sibling_idx, gadget_or_container)
		children = tuple(children)
		
		# set it back to parent container
		parent.setUiScriptGadgets(children)
		
		return None


	def isContainerHidden(self): # prob doesnt apply....
		'''Return whether a container or one of its containing containers is hidden.'''
		pass
	

	def isHidden(self): ## again not too sure
		
		'''Return whether the given gadget is itself hidden or lies inside a hidden container.'''
		pass
	

	def remove(self , name_or_gadget_or_indices):
		
		'''Remove a UI Script gadget from the window.
		
		   Note that if you remove a container, all of the gadgets in the container are also removed.'''

		# its a anme or gadget
		if not any([
		        type(name_or_gadget_or_indices)==type(()),
		        type(name_or_gadget_or_indices)==type([])
		       ]):

			remove_indices = self.findIndices(name_or_gadget_or_indices)

		# we have typle already
		else:
			remove_indices = name_or_gadget_or_indices
			
		# do the removal
		parent   = self.entryAtIndices(remove_indices[:-1])
		dead     = self.entryAtIndices(remove_indices)
		children = list(parent.uiScriptGadgets())

		children.remove(dead)
		children = tuple(children)
		
		# set it back to parent container
		parent.setUiScriptGadgets(children)
		
		return None

	
	def replace(self, name_or_gadget_or_indices, gadget_or_container):
		
		'''Replace a UI Script gadget inside this window with another UI Script gadget'''
		
		# its a anme or gadget
		if not any([
		        type(name_or_gadget_or_indices)==type(()),
		        type(name_or_gadget_or_indices)==type([])
		       ]):

			replace_indices = self.findIndices(name_or_gadget_or_indices)

		# we have typle already
		else:
			replace_indices = name_or_gadget_or_indices
			
		# do the replacement
		parent   = self.entryAtIndices(replace_indices[:-1])
		dead     = self.entryAtIndices(replace_indices)
		children = list(parent.uiScriptGadgets())
		dead_idx = children.index(dead)

		children[dead_idx] = gadget_or_container
		children = tuple(children)
		
		# set it back to parent container
		parent.setUiScriptGadgets(children)
		
		return None


	#def setToDialogScript(self): ## prob not possible currently
		#'''Set the contents of this window to match the parameters defined by the given dialog script string.'''
		#pass


	# ---------------------------------

	# manage window

	def createHouDialog(self, extend=True):
	  
		'''Create a Houdini Dialog from this DialogUiScriptWindow
		
		extend=True  bind instance specific methods for better control.
		             These do not alter other dialog instances or the Class
		             STRONGLY RECOMENDED'''
	  
		#import StringIO  
		import hou
		ui_script = self._printContainer()
	  
		ui_temp_file = tempfile.NamedTemporaryFile()
		ui_temp_file.file.write(ui_script)
		ui_temp_file.file.close()
		
		file_path = ui_temp_file.name
		#uiscript_file = StringIO.StringIO([self._printContainer()])
		#dialog = hou.ui.createDialog(uiscript_file.getvalue())
		
		# dialog obset is built but invisable
		dialog = hou.ui.createDialog(file_path)
		
		dialog = _extendHouDialog(self, dialog)
		
		dialog.setValue(self._name + '.val', 1)
		#uiscript_file.close()
		del(ui_temp_file)
		
		return dialog
	
	# ---------------------------------
	
	
	def _printExclusive(self):

		if self._exclusive == True:
		#if  self._printOkay(self._exclusive):
			result = 'EXLUSIVE'
		else : result = None

		return result
		

	def _printGadgetAttributes(self):

		gadget_attribs = [
			          self._printExclusive(),
			         ]
		result = [i for i in gadget_attribs if i !=None]
		return result
		

	def _printDeclarations(self):
	  
		# prints the container declaration with refs to children

		gadget_decl = self._name + '.win ='
		
		if  self._window_type == uiScriptWindowType.dialog:
		 	gadget_decl += ' DIALOG'
		else:
			gadget_decl += ' NORMAL_WINDOW'
			
		if self._title != None:
			gadget_decl += ' "' + self._title  +'"'
		
		elif  self._window_type == uiScriptWindowType.dialog:
			gadget_decl += ' "Houdini Dialog ..."'

		else: gadget_decl += ' "Houdini Window"'
		
		#if self._printOkay(self._icon):
		if self._icon != None:
			gadget_decl += ' ICON(' + self._icon +')'
		
		all_attribs = self._printAllAttributes()
		all_attribs = '    ' + all_attribs + '\n    VALUE(' + self._val + ')'

		gadget_refs = [x._name + '.gad;\n' for x in self._children]
		gadget_refs_string = '    ' + ('    '.join(gadget_refs) )
		
		gadget_decl += '\n{\n' + all_attribs + '\n\n' + gadget_refs_string + '}'

		# commands
		cmds        = self._printCmds()  ###???? not sure if these go here either

		# resulting string
		result = gadget_decl +'\n'+cmds
		
		return result

